@@ -13,7 +13,7 @@ std::map<wxString, int> CDirectoryListingParser::m_MonthNamesMap;
 #ifdef LISTDEBUG
 static char data[][150]={
 	"" // Has to be terminated with empty string
-};
+};a
 
 #endif
 
@@ -611,6 +611,20 @@ CDirectoryListingParser::CDirectoryListingParser(CControlSocket* pControlSocket,
 #endif
 		m_MonthNamesMap[_T("des")] = 12;
 
+		//Lithuanian
+		m_MonthNamesMap[_T("sau")] = 1;
+		m_MonthNamesMap[_T("vas")] = 2;
+		m_MonthNamesMap[_T("kov")] = 3;
+		m_MonthNamesMap[_T("bal")] = 4;
+		m_MonthNamesMap[_T("geg")] = 5;
+		m_MonthNamesMap[_T("bir")] = 6;
+		m_MonthNamesMap[_T("lie")] = 7;
+		m_MonthNamesMap[_T("rgp")] = 8;
+		m_MonthNamesMap[_T("rgs")] = 9;
+		m_MonthNamesMap[_T("spa")] = 10;
+		m_MonthNamesMap[_T("lap")] = 11;
+		m_MonthNamesMap[_T("grd")] = 12;
+
 		//There are more languages and thus month
 		//names, but as long as nobody reports a
 		//problem, I won't add them, there are way
@@ -669,9 +683,10 @@ CDirectoryListingParser::~CDirectoryListingParser()
 	delete m_prevLine;
 }
 
-void CDirectoryListingParser::ParseData(bool partial)
+bool CDirectoryListingParser::ParseData(bool partial)
 {
-	CLine *pLine = GetLine(partial);
+	bool error = false;
+	CLine *pLine = GetLine(partial, error);
 	while (pLine)
 	{
 		bool res = ParseLine(pLine, m_server.GetType(), false);
@@ -701,18 +716,24 @@ void CDirectoryListingParser::ParseData(bool partial)
 			m_prevLine = 0;
 			delete pLine;
 		}
-		pLine = GetLine(partial);
+		pLine = GetLine(partial, error);
 	};
+
+	return !error;
 }
 
 CDirectoryListing CDirectoryListingParser::Parse(const CServerPath &path)
 {
-	ParseData(false);
-
 	CDirectoryListing listing;
 	listing.path = path;
 	listing.m_firstListTime = CTimeEx::Now();
 
+	if (!ParseData(false))
+	{
+		listing.m_failed = true;
+		return listing;
+	}
+
 	if (!m_fileList.empty())
 	{
 		wxASSERT(m_entryList.empty());
@@ -723,11 +744,8 @@ CDirectoryListing CDirectoryListingParser::Parse(const CServerPath &path)
 		{
 			CDirentry entry;
 			entry.name = *iter;
-			entry.dir = false;
-			entry.hasTimestamp = CDirentry::timestamp_none;
-			entry.link = false;
+			entry.flags = 0;
 			entry.size = -1;
-			entry.unsure = false;
 			listing[i] = entry;
 		}
 	}
@@ -847,7 +865,7 @@ done:
 	if (entry.name == _T(".") || entry.name == _T(".."))
 		return true;
 
-	if (serverType == VMS && entry.dir)
+	if (serverType == VMS && entry.is_dir())
 	{
 		// Trim version information from directories
 		int pos = entry.name.Find(';', true);
@@ -857,7 +875,7 @@ done:
 
 	{
 		int offset = m_server.GetTimezoneOffset();
-		if (offset && entry.hasTimestamp >= CDirentry::timestamp_time)
+		if (offset && entry.has_time())
 		{
 			// Apply timezone offset
 			wxTimeSpan span(0, offset, 0, 0);
@@ -865,7 +883,6 @@ done:
 		}
 	}
 
-	entry.unsure = false;
 	m_entryList.push_back(entry);
 
 skip:
@@ -895,15 +912,13 @@ bool CDirectoryListingParser::ParseAsUnix(CLine *pLine, CDirentry &entry, bool e
 
 	entry.permissions = token.GetString();
 
+	entry.flags = 0;
+
 	if (chr == 'd' || chr == 'l')
-		entry.dir = true;
-	else
-		entry.dir = false;
+		entry.flags |= CDirentry::flag_dir;
 
 	if (chr == 'l')
-		entry.link = true;
-	else
-		entry.link = false;
+		entry.flags |= CDirentry::flag_link;
 
 	// Check for netware servers, which split the permissions into two parts
 	bool netware = false;
@@ -964,11 +979,10 @@ bool CDirectoryListingParser::ParseAsUnix(CLine *pLine, CDirentry &entry, bool e
 
 		if (expect_date)
 		{
+			entry.flags &= ~CDirentry::flag_timestamp_mask;
 			if (!ParseUnixDateTime(pLine, index, entry))
 				continue;
 		}
-		else
-			entry.hasTimestamp = CDirentry::timestamp_none;
 
 		// Get the filename
 		if (!pLine->GetToken(++index, token, 1))
@@ -983,7 +997,7 @@ bool CDirectoryListingParser::ParseAsUnix(CLine *pLine, CDirentry &entry, bool e
 			chr == '*')
 			entry.name.RemoveLast();
 
-		if (entry.link)
+		if (entry.is_link())
 		{
 			int pos;
 			if ((pos = entry.name.Find(_T(" -> "))) != -1)
@@ -993,7 +1007,7 @@ bool CDirectoryListingParser::ParseAsUnix(CLine *pLine, CDirentry &entry, bool e
 			}
 		}
 
-		if (entry.hasTimestamp >= CDirentry::timestamp_time)
+		if (entry.has_time())
 			entry.time.Add(m_timezoneOffset);
 
 		return true;
@@ -1234,7 +1248,10 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine *pLine, int &index, CDiren
 	entry.time = wxDateTime();
 	if (!VerifySetDate(entry.time, year, (wxDateTime::Month)(wxDateTime::Jan + month - 1), day, hour, minute))
 		return false;
-	entry.hasTimestamp = hasTime ? CDirentry::timestamp_time : CDirentry::timestamp_date;
+
+	entry.flags |= CDirentry::flag_timestamp_date;
+	if (hasTime)
+		entry.flags |= CDirentry::flag_timestamp_time;
 
 	return true;
 }
@@ -1394,7 +1411,7 @@ bool CDirectoryListingParser::ParseShortDate(CToken &token, CDirentry &entry, bo
 	entry.time = wxDateTime();
 	if (!VerifySetDate(entry.time, year, (wxDateTime::Month)(wxDateTime::Jan + month - 1), day))
 		return false;
-	entry.hasTimestamp = CDirentry::timestamp_date;
+	entry.flags |= CDirentry::flag_timestamp_date;
 
 	return true;
 }
@@ -1408,6 +1425,8 @@ bool CDirectoryListingParser::ParseAsDos(CLine *pLine, CDirentry &entry)
 	if (!pLine->GetToken(index, token))
 		return false;
 
+	entry.flags = 0;
+
 	if (!ParseShortDate(token, entry))
 		return false;
 
@@ -1425,7 +1444,7 @@ bool CDirectoryListingParser::ParseAsDos(CLine *pLine, CDirentry &entry)
 
 	if (token.GetString() == _T("<DIR>"))
 	{
-		entry.dir = true;
+		entry.flags |= CDirentry::flag_dir;
 		entry.size = -1;
 	}
 	else if (token.IsNumeric() || token.IsLeftNumeric())
@@ -1445,7 +1464,6 @@ bool CDirectoryListingParser::ParseAsDos(CLine *pLine, CDirentry &entry)
 			size += chr - '0';
 		}
 		entry.size = size;
-		entry.dir = false;
 	}
 	else
 		return false;
@@ -1455,12 +1473,11 @@ bool CDirectoryListingParser::ParseAsDos(CLine *pLine, CDirentry &entry)
 		return false;
 	entry.name = token.GetString();
 
-	entry.link = false;
 	entry.target = _T("");
 	entry.ownerGroup = _T("");
 	entry.permissions = _T("");
 
-	if (entry.hasTimestamp >= CDirentry::timestamp_time)
+	if (entry.has_time())
 		entry.time.Add(m_timezoneOffset);
 
 	return true;
@@ -1468,7 +1485,7 @@ bool CDirectoryListingParser::ParseAsDos(CLine *pLine, CDirentry &entry)
 
 bool CDirectoryListingParser::ParseTime(CToken &token, CDirentry &entry)
 {
-	if (entry.hasTimestamp == CDirentry::timestamp_none)
+	if (!entry.has_date())
 		return false;
 
 	int pos = token.Find(':');
@@ -1522,7 +1539,10 @@ bool CDirectoryListingParser::ParseTime(CToken &token, CDirentry &entry)
 
 	wxTimeSpan span(hour.GetLo(), minute.GetLo(), seconds.GetLo());
 	entry.time.Add(span);
-	entry.hasTimestamp = hasSeconds ? CDirentry::timestamp_seconds : CDirentry::timestamp_time;
+
+	entry.flags |= CDirentry::flag_timestamp_time;
+	if (hasSeconds)
+		entry.flags |= CDirentry::flag_timestamp_seconds;
 
 	return true;
 }
@@ -1542,11 +1562,9 @@ bool CDirectoryListingParser::ParseAsEplf(CLine *pLine, CDirentry &entry)
 
 	entry.name = token.GetString().Mid(pos + 1);
 
+	entry.flags = 0;
 	entry.ownerGroup = _T("");
 	entry.permissions = _T("");
-	entry.dir = false;
-	entry.link = false;
-	entry.hasTimestamp = CDirentry::timestamp_none;
 	entry.size = -1;
 
 	int fact = 1;
@@ -1568,7 +1586,7 @@ bool CDirectoryListingParser::ParseAsEplf(CLine *pLine, CDirentry &entry)
 		char type = token[fact];
 
 		if (type == '/')
-			entry.dir = true;
+			entry.flags |= CDirentry::flag_dir;
 		else if (type == 's')
 			entry.size = token.GetNumber(fact + 1, len - 1);
 		else if (type == 'm')
@@ -1578,7 +1596,7 @@ bool CDirectoryListingParser::ParseAsEplf(CLine *pLine, CDirentry &entry)
 				return false;
 			entry.time = wxDateTime((time_t)number.GetValue());
 
-			entry.hasTimestamp = CDirentry::timestamp_seconds;
+			entry.flags |= CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds;
 		}
 		else if (type == 'u' && len > 2 && token[fact + 1] == 'p')
 			entry.permissions = token.GetString().Mid(fact + 2, len - 2);
@@ -1619,19 +1637,18 @@ bool CDirectoryListingParser::ParseAsVms(CLine *pLine, CDirentry &entry)
 	if (pos == -1)
 		return false;
 
+	entry.flags = 0;
+
 	if (pos > 4 && token.GetString().Mid(pos - 4, 4) == _T(".DIR"))
 	{
-		entry.dir = true;
+		entry.flags |= CDirentry::flag_dir;
 		if (token.GetString().Mid(pos) == _T(";1"))
 			entry.name = token.GetString().Left(pos - 4);
 		else
 			entry.name = token.GetString().Left(pos - 4) + token.GetString().Mid(pos);
 	}
 	else
-	{
-		entry.dir = false;
 		entry.name = token.GetString();
-	}
 
 	// Some VMS servers escape special characters like additional dots with ^
 	entry.name = Unescape(entry.name, '^');
@@ -1766,7 +1783,7 @@ bool CDirectoryListingParser::ParseAsVms(CLine *pLine, CDirentry &entry)
 		}
 	}
 
-	if (entry.hasTimestamp >= CDirentry::timestamp_time)
+	if (entry.has_time())
 		entry.time.Add(m_timezoneOffset);
 
 	return true;
@@ -1796,6 +1813,8 @@ bool CDirectoryListingParser::ParseAsIbm(CLine *pLine, CDirentry &entry)
 	if (!pLine->GetToken(++index, token))
 		return false;
 
+	entry.flags = 0;
+
 	if (!ParseShortDate(token, entry))
 		return false;
 
@@ -1814,12 +1833,10 @@ bool CDirectoryListingParser::ParseAsIbm(CLine *pLine, CDirentry &entry)
 	if (token[token.GetLength() - 1] == '/')
 	{
 		entry.name.RemoveLast();
-		entry.dir = true;
+		entry.flags |= CDirentry::flag_dir;
 	}
-	else
-		entry.dir = false;
-
-	if (entry.hasTimestamp >= CDirentry::timestamp_time)
+	
+	if (entry.has_time())
 		entry.time.Add(m_timezoneOffset);
 
 	return true;
@@ -1842,15 +1859,15 @@ bool CDirectoryListingParser::ParseOther(CLine *pLine, CDirentry &entry)
 	if (!pLine->GetToken(++index, token))
 		return false;
 
+	entry.flags = 0;
+
 	// If token is a number, than it's the numerical Unix style format,
 	// else it's the VShell, OS/2 or nortel.VxWorks format
 	if (token.IsNumeric())
 	{
 		entry.permissions = firstToken.GetString();
 		if (firstToken.GetLength() >= 2 && firstToken[1] == '4')
-			entry.dir = true;
-		else
-			entry.dir = false;
+			entry.flags |= CDirentry::flag_dir;
 
 		entry.ownerGroup += token.GetString();
 
@@ -1877,7 +1894,7 @@ bool CDirectoryListingParser::ParseOther(CLine *pLine, CDirentry &entry)
 			return false;
 		entry.time = wxDateTime((time_t)number.GetValue());
 
-		entry.hasTimestamp = CDirentry::timestamp_seconds;
+		entry.flags |= CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds;
 
 		// Get filename
 		if (!pLine->GetToken(++index, token, true))
@@ -1885,7 +1902,6 @@ bool CDirectoryListingParser::ParseOther(CLine *pLine, CDirentry &entry)
 
 		entry.name = token.GetString();
 
-		entry.link = false;
 		entry.target = _T("");
 	}
 	else
@@ -1903,12 +1919,11 @@ bool CDirectoryListingParser::ParseOther(CLine *pLine, CDirentry &entry)
 		if (!GetMonthFromName(dateMonth, month))
 		{
 			// OS/2 or nortel.VxWorks
-			entry.dir = false;
 			int skippedCount = 0;
 			do
 			{
 				if (token.GetString() == _T("DIR"))
-					entry.dir = true;
+					entry.flags |= CDirentry::flag_dir;
 				else if (token.Find(_T("-/.")) != -1)
 					break;
 
@@ -1937,7 +1952,7 @@ bool CDirectoryListingParser::ParseOther(CLine *pLine, CDirentry &entry)
 			MakeLowerAscii(type);
 			if (!skippedCount && type == _T("<dir>"))
 			{
-				entry.dir = true;
+				entry.flags |= CDirentry::flag_dir;
 				entry.name = entry.name.Left(entry.name.Length() - 5);
 				while (entry.name.Last() == ' ')
 					entry.name.RemoveLast();
@@ -1973,7 +1988,7 @@ bool CDirectoryListingParser::ParseOther(CLine *pLine, CDirentry &entry)
 			if (!VerifySetDate(entry.time, year.GetLo(), (wxDateTime::Month)(month - 1), day.GetLo()))
 				return false;
 
-			entry.hasTimestamp = CDirentry::timestamp_date;
+			entry.flags |= CDirentry::flag_timestamp_date;
 
 			// Get time
 			if (!pLine->GetToken(++index, token))
@@ -1990,25 +2005,22 @@ bool CDirectoryListingParser::ParseOther(CLine *pLine, CDirentry &entry)
 			char chr = token[token.GetLength() - 1];
 			if (chr == '/' || chr == '\\')
 			{
-				entry.dir = true;
+				entry.flags |= CDirentry::flag_dir;
 				entry.name.RemoveLast();
 			}
-			else
-				entry.dir = false;
 		}
-		entry.link = false;
 		entry.target = _T("");
 		entry.ownerGroup = _T("");
 		entry.permissions = _T("");
 
-		if (entry.hasTimestamp >= CDirentry::timestamp_time)
+		if (entry.has_time())
 			entry.time.Add(m_timezoneOffset);
 	}
 
 	return true;
 }
 
-void CDirectoryListingParser::AddData(char *pData, int len)
+bool CDirectoryListingParser::AddData(char *pData, int len)
 {
 	t_list item;
 	item.p = pData;
@@ -2016,10 +2028,10 @@ void CDirectoryListingParser::AddData(char *pData, int len)
 
 	m_DataList.push_back(item);
 
-	ParseData(true);
+	return ParseData(true);
 }
 
-void CDirectoryListingParser::AddLine(const wxChar* pLine)
+bool CDirectoryListingParser::AddLine(const wxChar* pLine)
 {
 	if (m_pControlSocket)
 		m_pControlSocket->LogMessageRaw(RawList, pLine);
@@ -2028,7 +2040,7 @@ void CDirectoryListingParser::AddLine(const wxChar* pLine)
 		pLine++;
 
 	if (!*pLine)
-		return;
+		return false;
 
 	const int len = wxStrlen(pLine);
 
@@ -2039,9 +2051,11 @@ void CDirectoryListingParser::AddLine(const wxChar* pLine)
 	CLine line(p, len);
 
 	ParseLine(&line, m_server.GetType(), false);
+
+	return true;
 }
 
-CLine *CDirectoryListingParser::GetLine(bool breakAtEnd /*=false*/)
+CLine *CDirectoryListingParser::GetLine(bool breakAtEnd /*=false*/, bool &error)
 {
 	while (!m_DataList.empty())
 	{
@@ -2088,6 +2102,12 @@ CLine *CDirectoryListingParser::GetLine(bool breakAtEnd /*=false*/)
 				iter++;
 				if (iter == m_DataList.end())
 				{
+					if (reslen > 10000)
+					{
+						m_pControlSocket->LogMessage(::Error, _("Received a line exceeding 10000 characters, aborting."));
+						error = true;
+						return 0;
+					}
 					if (breakAtEnd)
 						return 0;
 					break;
@@ -2096,6 +2116,13 @@ CLine *CDirectoryListingParser::GetLine(bool breakAtEnd /*=false*/)
 				currentOffset = 0;
 			}
 		}
+
+		if (reslen > 10000)
+		{
+			m_pControlSocket->LogMessage(::Error, _("Received a line exceeding 10000 characters, aborting."));
+			error = true;
+			return 0;
+		}
 		m_currentOffset = currentOffset;
 
 		// Reslen is now the length of the line, including any terminating whitespace
@@ -2190,7 +2217,9 @@ bool CDirectoryListingParser::ParseAsWfFtp(CLine *pLine, CDirentry &entry)
 
 	entry.size = token.GetNumber();
 
-	// Parse daste
+	entry.flags = 0;
+
+	// Parse date
 	if (!pLine->GetToken(index++, token))
 		return false;
 
@@ -2211,12 +2240,10 @@ bool CDirectoryListingParser::ParseAsWfFtp(CLine *pLine, CDirentry &entry)
 	if (!ParseTime(token, entry))
 		return false;
 
-	entry.dir = false;
-	entry.link = false;
 	entry.ownerGroup = _T("");
 	entry.permissions = _T("");
 
-	if (entry.hasTimestamp >= CDirentry::timestamp_time)
+	if (entry.has_time())
 		entry.time.Add(m_timezoneOffset);
 
 	return true;
@@ -2239,9 +2266,8 @@ bool CDirectoryListingParser::ParseAsIBM_MVS(CLine *pLine, CDirentry &entry)
 	if (!pLine->GetToken(index++, token))
 		return false;
 
-	if (token.GetString() == _T("**NONE**"))
-		entry.hasTimestamp = CDirentry::timestamp_none;
-	else if (!ParseShortDate(token, entry))
+	entry.flags = 0;
+	if (token.GetString() != _T("**NONE**") && !ParseShortDate(token, entry))
 	{
 		// Perhaps of the following type:
 		// TSO004 3390 VSAM FOO.BAR
@@ -2256,11 +2282,8 @@ bool CDirectoryListingParser::ParseAsIBM_MVS(CLine *pLine, CDirentry &entry)
 			return false;
 
 		entry.size = -1;
-		entry.dir = false;
 		entry.ownerGroup = _T("");
 		entry.permissions = _T("");
-		entry.hasTimestamp = CDirentry::timestamp_none;
-		entry.link = false;
 
 		return true;
 	}
@@ -2276,7 +2299,7 @@ bool CDirectoryListingParser::ParseAsIBM_MVS(CLine *pLine, CDirentry &entry)
 	// used
 	if (!pLine->GetToken(index++, token))
 		return false;
-	if (token.IsNumeric())
+	if (token.IsNumeric() || token.GetString() == _T("????"))
 	{
 		// recfm
 		if (!pLine->GetToken(index++, token))
@@ -2308,14 +2331,11 @@ bool CDirectoryListingParser::ParseAsIBM_MVS(CLine *pLine, CDirentry &entry)
 
 	if (token.GetString() == _T("PO"))
 	{
-		entry.dir = true;
+		entry.flags |= CDirentry::flag_dir;
 		entry.size = -1;
 	}
 	else
-	{
-		entry.dir = false;
 		entry.size = 100;
-	}
 
 	// name of dataset or sequential file
 	if (!pLine->GetToken(index++, token, true))
@@ -2325,7 +2345,6 @@ bool CDirectoryListingParser::ParseAsIBM_MVS(CLine *pLine, CDirentry &entry)
 
 	entry.ownerGroup = _T("");
 	entry.permissions = _T("");
-	entry.link = false;
 
 	return true;
 }
@@ -2344,6 +2363,8 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_PDS(CLine *pLine, CDirentry &entry)
 	if (!pLine->GetToken(index++, token))
 		return false;
 
+	entry.flags = 0;
+
 	// creation date
 	if (!pLine->GetToken(index++, token))
 		return false;
@@ -2385,12 +2406,10 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_PDS(CLine *pLine, CDirentry &entry)
 	if (!pLine->GetToken(index++, token, true))
 		return false;
 
-	entry.dir = false;
 	entry.ownerGroup = _T("");
 	entry.permissions = _T("");
-	entry.link = false;
 
-	if (entry.hasTimestamp >= CDirentry::timestamp_time)
+	if (entry.has_time())
 		entry.time.Add(m_timezoneOffset);
 
 	return true;
@@ -2414,12 +2433,10 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_Migrated(CLine *pLine, CDirentry &e
 
 	entry.name = token.GetString();
 
-	entry.dir = false;
-	entry.link = false;
+	entry.flags = 0;
 	entry.ownerGroup = _T("");
 	entry.permissions = _T("");
 	entry.size = -1;
-	entry.hasTimestamp = CDirentry::timestamp_none;
 
 	if (pLine->GetToken(++index, token))
 		return false;
@@ -2436,12 +2453,10 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_PDS2(CLine *pLine, CDirentry &entry
 
 	entry.name = token.GetString();
 
-	entry.dir = false;
-	entry.link = false;
+	entry.flags = 0;
 	entry.ownerGroup = _T("");
 	entry.permissions = _T("");
 	entry.size = -1;
-	entry.hasTimestamp = CDirentry::timestamp_none;
 
 	if (!pLine->GetToken(++index, token))
 		return true;
@@ -2512,13 +2527,11 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_Tape(CLine *pLine, CDirentry &entry
 		return false;
 
 	entry.name = token.GetString();
-	entry.dir = false;
-	entry.link = false;
+	entry.flags = 0;
 	entry.ownerGroup = _T("");
 	entry.permissions = _T("");
 	entry.size = -1;
-	entry.hasTimestamp = CDirentry::timestamp_none;
-
+	
 	if (pLine->GetToken(index++, token))
 		return false;
 
@@ -2631,10 +2644,8 @@ int CDirectoryListingParser::ParseAsMlsd(CLine *pLine, CDirentry &entry)
 	if (facts.IsEmpty())
 		return 0;
 
-	entry.dir = false;
-	entry.link = false;
+	entry.flags = 0;
 	entry.size = -1;
-	entry.hasTimestamp = CDirentry::timestamp_none;
 	entry.ownerGroup = _T("");
 	entry.permissions = _T("");
 
@@ -2661,11 +2672,10 @@ int CDirectoryListingParser::ParseAsMlsd(CLine *pLine, CDirentry &entry)
 		if (factname == _T("type"))
 		{
 			if (!value.CmpNoCase(_T("dir")))
-				entry.dir = true;
+				entry.flags |= CDirentry::flag_dir;
 			else if (!value.Left(13).CmpNoCase(_T("OS.unix=slink")))
 			{
-				entry.dir = true;
-				entry.link = true;
+				entry.flags |= CDirentry::flag_dir | CDirentry::flag_link;
 				if (value[13] == ':' && value[14] != 0)
 					entry.target = value.Mid(14);
 			}
@@ -2686,7 +2696,7 @@ int CDirectoryListingParser::ParseAsMlsd(CLine *pLine, CDirentry &entry)
 			}
 		}
 		else if (factname == _T("modify") ||
-			(entry.hasTimestamp == CDirentry::timestamp_none && factname == _T("create")))
+			(!entry.has_date() && factname == _T("create")))
 		{
 			wxDateTime dateTime;
 			const wxChar* time = dateTime.ParseFormat(value, _T("%Y%m%d"));
@@ -2698,10 +2708,10 @@ int CDirectoryListingParser::ParseAsMlsd(CLine *pLine, CDirentry &entry)
 			{
 				if (!dateTime.ParseFormat(time, _T("%H%M%S"), dateTime))
 					return 0;
-				entry.hasTimestamp = CDirentry::timestamp_seconds;
+				entry.flags |= CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds;
 			}
 			else
-				entry.hasTimestamp = CDirentry::timestamp_date;
+				entry.flags |= CDirentry::flag_timestamp_date;
 
 			entry.time = dateTime.FromTimezone(wxDateTime::GMT0);
 		}
@@ -2775,6 +2785,8 @@ bool CDirectoryListingParser::ParseAsOS9(CLine *pLine, CDirentry &entry)
 
 	entry.ownerGroup = token.GetString();
 
+	entry.flags = 0;
+
 	// Get date
 	if (!pLine->GetToken(index++, token))
 		return false;
@@ -2792,7 +2804,8 @@ bool CDirectoryListingParser::ParseAsOS9(CLine *pLine, CDirentry &entry)
 
 	entry.permissions = token.GetString();
 
-	entry.dir = token[0] == 'd';
+	if (token[0] == 'd')
+		entry.flags |= CDirentry::flag_dir;
 
 	// Unused token
 	if (!pLine->GetToken(index++, token))
@@ -2813,8 +2826,6 @@ bool CDirectoryListingParser::ParseAsOS9(CLine *pLine, CDirentry &entry)
 
 	entry.name = token.GetString();
 
-	entry.link = false;
-
 	return true;
 }
 
@@ -2882,6 +2893,8 @@ bool CDirectoryListingParser::ParseAsZVM(CLine* pLine, CDirentry &entry)
 	if (!token.IsNumeric())
 		return false;
 
+	entry.flags = 0;
+
 	// Date
 	if (!pLine->GetToken(++index, token))
 		return false;
@@ -2906,12 +2919,10 @@ bool CDirectoryListingParser::ParseAsZVM(CLine* pLine, CDirentry &entry)
 	if (pLine->GetToken(++index, token))
 		return false;
 
-	entry.dir = false;
-	entry.link = false;
 	entry.permissions = _T("");
 	entry.target = _T("");
 
-	if (entry.hasTimestamp >= CDirentry::timestamp_time)
+	if (entry.has_time())
 		entry.time.Add(m_timezoneOffset);
 
 	return true;
@@ -2942,6 +2953,8 @@ bool CDirectoryListingParser::ParseAsHPNonstop(CLine *pLine, CDirentry &entry)
 
 	entry.size = token.GetNumber();
 
+	entry.flags = 0;
+
 	// Date
 	if (!pLine->GetToken(++index, token))
 		return false;
@@ -2976,9 +2989,6 @@ bool CDirectoryListingParser::ParseAsHPNonstop(CLine *pLine, CDirentry &entry)
 	if (pLine->GetToken(++index, token))
 		return false;
 
-	entry.dir = false;
-	entry.link = false;
-		
 	return true;
 }
 
