@@ -408,9 +408,7 @@ int CSftpControlSocket::Connect(const CServer &server)
 		m_useUTF8 = true;
 	}
 
-
-	if (m_pCurrentServer)
-		delete m_pCurrentServer;
+	delete m_pCurrentServer;
 	m_pCurrentServer = new CServer(server);
 
 	CSftpConnectOpData* pData = new CSftpConnectOpData;
@@ -870,9 +868,10 @@ bool CSftpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 			CHostKeyNotification *pHostKeyNotification = reinterpret_cast<CHostKeyNotification *>(pNotification);
 			wxString show;
 			if (requestId == reqId_hostkey)
-				show = _("Trust new Hostkey: ");
+				show = _("Trust new Hostkey:");
 			else
-				show = _("Trust changed Hostkey: ");
+				show = _("Trust changed Hostkey:");
+			show += ' ';
 			if (!pHostKeyNotification->m_trust)
 			{
 				Send(_T(""), show + _("No"));
@@ -1057,12 +1056,12 @@ int CSftpControlSocket::ListParseResponse(bool successful, const wxString& reply
 				if (date.IsValid())
 				{
 					date.MakeTimezone(wxDateTime::GMT0);
-					wxASSERT(pData->directoryListing[pData->mtime_index].hasTimestamp != CDirentry::timestamp_none);
+					wxASSERT(pData->directoryListing[pData->mtime_index].has_date());
 					wxDateTime listTime = pData->directoryListing[pData->mtime_index].time;
 					listTime -= wxTimeSpan(0, m_pCurrentServer->GetTimezoneOffset(), 0);
 
 					int serveroffset = (date - listTime).GetSeconds().GetLo();
-					if (pData->directoryListing[pData->mtime_index].hasTimestamp != CDirentry::timestamp_seconds)
+					if (!pData->directoryListing[pData->mtime_index].has_seconds())
 					{
 						// Round offset to full minutes
 						if (serveroffset < 0)
@@ -1083,7 +1082,7 @@ int CSftpControlSocket::ListParseResponse(bool successful, const wxString& reply
 					for (int i = 0; i < count; i++)
 					{
 						CDirentry& entry = pData->directoryListing[i];
-						if (entry.hasTimestamp < CDirentry::timestamp_time)
+						if (!entry.has_time())
 							continue;
 
 						entry.time += span;
@@ -1700,18 +1699,17 @@ int CSftpControlSocket::FileTransferSubcommandResult(int prevResult)
 			}
 			else
 			{
-				if (entry.unsure)
+				if (entry.is_unsure())
 					pData->opState = filetransfer_waitlist;
 				else
 				{
 					if (matchedCase)
 					{
 						pData->remoteFileSize = entry.size.GetLo() + ((wxFileOffset)entry.size.GetHi() << 32);
-						if (entry.hasTimestamp != CDirentry::timestamp_none)
+						if (entry.has_date())
 							pData->fileTime = entry.time;
 
-						if (pData->download &&
-							entry.hasTimestamp < CDirentry::timestamp_time &&
+						if (pData->download && !entry.has_time() &&
 							m_pEngine->GetOptions()->GetOptionVal(OPTION_PRESERVE_TIMESTAMPS))
 						{
 							pData->opState = filetransfer_mtime;
@@ -1767,14 +1765,13 @@ int CSftpControlSocket::FileTransferSubcommandResult(int prevResult)
 			}
 			else
 			{
-				if (matchedCase && !entry.unsure)
+				if (matchedCase && !entry.is_unsure())
 				{
 					pData->remoteFileSize = entry.size.GetLo() + ((wxFileOffset)entry.size.GetHi() << 32);
-					if (entry.hasTimestamp != CDirentry::timestamp_none)
+					if (!entry.has_date())
 						pData->fileTime = entry.time;
 
-					if (pData->download &&
-						entry.hasTimestamp < CDirentry::timestamp_time &&
+					if (pData->download && !entry.has_time() &&
 						m_pEngine->GetOptions()->GetOptionVal(OPTION_PRESERVE_TIMESTAMPS))
 					{
 						pData->opState = filetransfer_mtime;
@@ -2769,10 +2766,10 @@ int CSftpControlSocket::ListCheckTimezoneDetection()
 		const int count = pData->directoryListing.GetCount();
 		for (int i = 0; i < count; i++)
 		{
-			if (pData->directoryListing[i].hasTimestamp < CDirentry::timestamp_time)
+			if (!pData->directoryListing[i].has_time())
 				continue;
 
-			if (pData->directoryListing[i].link)
+			if (pData->directoryListing[i].is_link())
 				continue;
 
 			pData->opState = list_mtime;
