@@ -34,6 +34,16 @@
   #define EAI_ADDRFAMILY EAI_FAMILY
 #endif
 
+// Union for strict aliasing-safe casting between
+// the different address types
+union sockaddr_u
+{
+	struct sockaddr_storage storage;
+	struct sockaddr sockaddr;
+	struct sockaddr_in in4;
+	struct sockaddr_in6 in6;
+};
+
 // --------------------------
 // Windows 2000 compatibility
 // --------------------------
@@ -358,16 +368,13 @@ public:
 		if (!already_locked)
 			m_sync.Lock();
 		m_pSocket = pSocket;
-		if (m_pHost)
-		{
-			delete [] m_pHost;
-			m_pHost = 0;
-		}
-		if (m_pPort)
-		{
-			delete [] m_pPort;
-			m_pPort = 0;
-		}
+
+		delete [] m_pHost;
+		m_pHost = 0;
+
+		delete [] m_pPort;
+		m_pPort = 0;
+
 		m_waiting = 0;
 		if (!already_locked)
 			m_sync.Unlock();
@@ -376,10 +383,8 @@ public:
 	int Connect()
 	{
 		wxASSERT(m_pSocket);
-		if (m_pHost)
-			delete [] m_pHost;
-		if (m_pPort)
-			delete [] m_pPort;
+		delete [] m_pHost;
+		delete [] m_pPort;
 
 		const wxWX2MBbuf buf = m_pSocket->m_host.mb_str();
 		if (!buf)
@@ -461,7 +466,12 @@ public:
 		WSASetEvent(m_sync_event);
 #else
 		char tmp = 0;
-		write(m_pipe[1], &tmp, 1);
+
+		int ret;
+		do
+		{
+			ret = write(m_pipe[1], &tmp, 1);
+		} while (ret == -1 && errno == EINTR);
 #endif
 		if (!already_locked)
 			m_sync.Unlock();
@@ -853,7 +863,8 @@ protected:
 			if (res > 0 && FD_ISSET(m_pipe[0], &readfds))
 			{
 				char buffer[100];
-				read(m_pipe[0], buffer, 100);
+				int damn_spurious_warning = read(m_pipe[0], buffer, 100);
+				(void)damn_spurious_warning; // We do not care about return value and this is definitely correct!
 			}
 
 			if (m_quit || !m_pSocket || m_pSocket->m_fd == -1)
@@ -1613,13 +1624,13 @@ wxString CSocket::GetPeerIP(bool strip_zone_index /*=false*/) const
 
 enum CSocket::address_family CSocket::GetAddressFamily() const
 {
-	struct sockaddr_storage addr;
+	sockaddr_u addr;
 	socklen_t addr_len = sizeof(addr);
-	int res = getsockname(m_fd, (sockaddr*)&addr, &addr_len);
+	int res = getsockname(m_fd, &addr.sockaddr, &addr_len);
 	if (res)
 		return unspec;
 
-	switch (((sockaddr*)&addr)->sa_family)
+	switch (addr.sockaddr.sa_family)
 	{
 	case AF_INET:
 		return ipv4;
@@ -1760,9 +1771,9 @@ int CSocket::Listen(enum address_family family, int port /*=0*/)
 
 int CSocket::GetLocalPort(int& error)
 {
-	struct sockaddr_storage addr;
+	sockaddr_u addr;
 	socklen_t addr_len = sizeof(addr);
-	error = getsockname(m_fd, (sockaddr*)&addr, &addr_len);
+	error = getsockname(m_fd, &addr.sockaddr, &addr_len);
 	if (error)
 	{
 #ifdef __WXMSW__
@@ -1771,16 +1782,10 @@ int CSocket::GetLocalPort(int& error)
 		return -1;
 	}
 
-	if (addr.ss_family == AF_INET)
-	{
-		struct sockaddr_in* addr_v4 = (sockaddr_in*)&addr;
-		return ntohs(addr_v4->sin_port);
-	}
-	else if (addr.ss_family == AF_INET6)
-	{
-		struct sockaddr_in6* addr_v6 = (sockaddr_in6*)&addr;
-		return ntohs(addr_v6->sin6_port);
-	}
+	if (addr.storage.ss_family == AF_INET)
+		return ntohs(addr.in4.sin_port);
+	else if (addr.storage.ss_family == AF_INET6)
+		return ntohs(addr.in6.sin6_port);
 
 	error = EINVAL;
 	return -1;
@@ -1788,9 +1793,9 @@ int CSocket::GetLocalPort(int& error)
 
 int CSocket::GetRemotePort(int& error)
 {
-	struct sockaddr_storage addr;
+	sockaddr_u addr;		
 	socklen_t addr_len = sizeof(addr);
-	error = getpeername(m_fd, (sockaddr*)&addr, &addr_len);
+	error = getpeername(m_fd, &addr.sockaddr, &addr_len);
 	if (error)
 	{
 #ifdef __WXMSW__
@@ -1799,16 +1804,10 @@ int CSocket::GetRemotePort(int& error)
 		return -1;
 	}
 
-	if (addr.ss_family == AF_INET)
-	{
-		struct sockaddr_in* addr_v4 = (sockaddr_in*)&addr;
-		return ntohs(addr_v4->sin_port);
-	}
-	else if (addr.ss_family == AF_INET6)
-	{
-		struct sockaddr_in6* addr_v6 = (sockaddr_in6*)&addr;
-		return ntohs(addr_v6->sin6_port);
-	}
+	if (addr.storage.ss_family == AF_INET)
+		return ntohs(addr.in4.sin_port);
+	else if (addr.storage.ss_family == AF_INET6)
+		return ntohs(addr.in6.sin6_port);
 
 	error = EINVAL;
 	return -1;
