@@ -275,7 +275,7 @@ CLocalListView::CLocalListView(wxWindow* pParent, CState *pState, CQueueView *pQ
 	m_pVolumeEnumeratorThread = 0;
 #endif
 
-	InitHeaderImageList();
+	InitHeaderSortImageList();
 
 	SetDropTarget(new CLocalListViewDropTarget(this));
 
@@ -399,7 +399,7 @@ regular_dir:
 			data.hasTime = data.lastModified.IsValid();
 
 			m_fileData.push_back(data);
-			if (!filter.FilenameFiltered(data.name, dirname, data.dir, data.size, true, data.attributes))
+			if (!filter.FilenameFiltered(data.name, dirname, data.dir, data.size, true, data.attributes, data.hasTime ? &data.lastModified : 0))
 			{
 				if (data.dir)
 					totalDirCount++;
@@ -580,11 +580,9 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 		return;
 	}
 
-	wxFileName fn(m_dir, data->name);
-
 	const bool queue_only = action == 1;
 
-	m_pQueue->QueueFile(queue_only, false, fn.GetFullPath(), data->name, path, *pServer, data->size);
+	m_pQueue->QueueFile(queue_only, false, CLocalPath(m_dir), data->name, data->name, path, *pServer, data->size);
 	m_pQueue->QueueFile_Finish(true);
 }
 
@@ -959,8 +957,7 @@ public:
 
 		CMP(CmpDir, data1, data2);
 
-		if (!data1.dir)
-			CMP(CmpSize, data1, data2)
+		CMP(CmpSize, data1, data2)
 
 		CMP_LESS(CmpName, data1, data2);
 	}
@@ -1167,15 +1164,13 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 		{
 			path.ChangePath(data->name);
 
-			wxFileName fn(m_dir, _T(""));
-			fn.AppendDir(data->name);
-			m_pQueue->QueueFolder(event.GetId() == XRCID("ID_ADDTOQUEUE"), false, fn.GetPath(), path, *pServer);
+			CLocalPath localPath(m_dir);
+			localPath.AddSegment(data->name);
+			m_pQueue->QueueFolder(event.GetId() == XRCID("ID_ADDTOQUEUE"), false, localPath, path, *pServer);
 		}
 		else
 		{
-			wxFileName fn(m_dir, data->name);
-
-			m_pQueue->QueueFile(queue_only, false, fn.GetFullPath(), data->name, path, *pServer, data->size);
+			m_pQueue->QueueFile(queue_only, false, CLocalPath(m_dir), data->name, data->name, path, *pServer, data->size);
 			added = true;
 		}
 	}
@@ -1276,7 +1271,7 @@ void CLocalListView::OnKeyDown(wxKeyEvent& event)
 #endif
 
 	const int code = event.GetKeyCode();
-	if (code == WXK_DELETE)
+	if (code == WXK_DELETE || code == WXK_NUMPAD_DELETE)
 	{
 		if (GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) == -1)
 		{
@@ -1440,7 +1435,7 @@ void CLocalListView::ApplyCurrentFilter()
 		const CLocalFileData& data = m_fileData[i];
 		if (data.flags == fill)
 			continue;
-		if (filter.FilenameFiltered(data.name, m_dir, data.dir, data.size, true, data.attributes))
+		if (filter.FilenameFiltered(data.name, m_dir, data.dir, data.size, true, data.attributes, data.hasTime ? &data.lastModified : 0))
 		{
 			hidden++;
 			continue;
@@ -1677,7 +1672,7 @@ void CLocalListView::RefreshFile(const wxString& file)
 	data.hasTime = data.lastModified.IsValid();
 
 	CFilterManager filter;
-	if (filter.FilenameFiltered(data.name, m_dir, data.dir, data.size, true, data.attributes))
+	if (filter.FilenameFiltered(data.name, m_dir, data.dir, data.size, true, data.attributes, data.hasTime ? &data.lastModified : 0))
 		return;
 
 	CancelLabelEdit();
@@ -2237,13 +2232,13 @@ void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 		wxString drive = iter->volume;
 
 		unsigned int item, index;
-		for (item = 1; item < m_indexMapping.size(); item++)
+		for (item = 1; item < m_indexMapping.size(); ++item)
 		{
 			index = m_indexMapping[item];
 			if (m_fileData[index].name == drive || m_fileData[index].name.Left(drive.Len() + 1) == drive + _T(" "))
 				break;
 		}
-		if (item == m_indexMapping.size())
+		if (item >= m_indexMapping.size())
 			continue;
 
 		m_fileData[index].label = drive + _T(" (") + iter->volumeName + _T(")");
