@@ -215,7 +215,7 @@ public:
 		m_processing_entries = false;
 
 		t_dirPair* pair = new t_dirPair;
-		pair->localPath = pFolderItem->GetLocalPath().c_str();
+		pair->localPath = pFolderItem->GetLocalPath().GetPath().c_str();
 		pair->remotePath.SetSafePath(pFolderItem->GetRemotePath().GetSafePath().c_str());
 		m_dirsToCheck.push_back(pair);
 	}
@@ -252,14 +252,14 @@ public:
 		CServerPath remotePath;
 	};
 
-	void ProcessDirectory(const wxString& localPath, CServerPath remotePath, const wxString& name)
+	void ProcessDirectory(const CLocalPath& localPath, CServerPath remotePath, const wxString& name)
 	{
 		wxMutexLocker locker(m_sync);
 
 		t_dirPair* pair = new t_dirPair;
 
 		{
-			pair->localPath = (localPath + CLocalFileSystem::path_separator + name).c_str();
+			pair->localPath = (localPath.GetPath() + name).c_str();
 
 			remotePath.AddSegment(name);
 			pair->remotePath.SetSafePath(remotePath.GetSafePath().c_str());
@@ -519,7 +519,8 @@ CQueueView::~CQueueView()
 #endif
 }
 
-bool CQueueView::QueueFile(const bool queueOnly, const bool download, const wxString& localFile,
+bool CQueueView::QueueFile(const bool queueOnly, const bool download,
+						   const CLocalPath& localPath, const wxString& localFile,
 						   const wxString& remoteFile, const CServerPath& remotePath,
 						   const CServer& server, const wxLongLong size, enum CEditHandler::fileType edit /*=CEditHandler::none*/)
 {
@@ -529,14 +530,18 @@ bool CQueueView::QueueFile(const bool queueOnly, const bool download, const wxSt
 	if (localFile == _T("") || remotePath.IsEmpty())
 	{
 		if (download)
-			fileItem = new CFolderItem(pServerItem, queueOnly, localFile);
+		{
+			CLocalPath p(localPath);
+			p.AddSegment(localFile);
+			fileItem = new CFolderItem(pServerItem, queueOnly, p);
+		}
 		else
 			fileItem = new CFolderItem(pServerItem, queueOnly, remotePath, remoteFile);
 		wxASSERT(edit == CEditHandler::none);
 	}
 	else
 	{
-		fileItem = new CFileItem(pServerItem, queueOnly, download, localFile, remoteFile, remotePath, size);
+		fileItem = new CFileItem(pServerItem, queueOnly, download, localPath, localFile, remoteFile, remotePath, size);
 		if (download)
 			fileItem->m_transferSettings.binary = !CAutoAsciiFiles::TransferRemoteAsAscii(remoteFile, remotePath.GetType());
 		else
@@ -577,7 +582,7 @@ void CQueueView::QueueFile_Finish(const bool start)
 		RefreshListOnly(false);
 }
 
-bool CQueueView::QueueFiles(const bool queueOnly, const wxString& localPath, const CRemoteDataObject& dataObject)
+bool CQueueView::QueueFiles(const bool queueOnly, const CLocalPath& localPath, const CRemoteDataObject& dataObject)
 {
 	CServerItem* pServerItem = CreateServerItem(dataObject.GetServer());
 
@@ -589,7 +594,7 @@ bool CQueueView::QueueFiles(const bool queueOnly, const wxString& localPath, con
 			continue;
 
 		CFileItem* fileItem;
-		fileItem = new CFileItem(pServerItem, queueOnly, true, localPath + ReplaceInvalidCharacters(iter->name), iter->name, dataObject.GetServerPath(), iter->size);
+		fileItem = new CFileItem(pServerItem, queueOnly, true, localPath, ReplaceInvalidCharacters(iter->name), iter->name, dataObject.GetServerPath(), iter->size);
 		fileItem->m_transferSettings.binary = !CAutoAsciiFiles::TransferRemoteAsAscii(iter->name, dataObject.GetServerPath().GetType());
 
 		InsertItem(pServerItem, fileItem);
@@ -719,7 +724,7 @@ void CQueueView::ProcessNotification(t_EngineData* pEngineData, CNotification* p
 					pEngineData->pItem->GetType() == QueueItemType_File)
 				{
 					CFileItem* pItem = (CFileItem*)pEngineData->pItem;
-					pItem->m_madeProgress = true;
+					pItem->set_made_progress(true);
 				}
 				pEngineData->pStatusLineCtrl->SetTransferStatus(pStatus);
 			}
@@ -830,7 +835,6 @@ bool CQueueView::TryStartNextTransfer()
 		CFileItem* fileItem;
 		CServerItem* serverItem;
 		t_EngineData* pEngineData;
-		bool browsingConnection;
 	} bestMatch = {0};
 
 	// Find inactive file. Check all servers for
@@ -847,11 +851,12 @@ bool CQueueView::TryStartNextTransfer()
 
 		while (newFileItem && newFileItem->Download() && newFileItem->GetType() == QueueItemType_Folder)
 		{
-			wxFileName fn(newFileItem->GetLocalFile(), _T(""));
-			wxFileName::Mkdir(fn.GetPath(), 0777, wxPATH_MKDIR_FULL);
+			CLocalPath localPath(newFileItem->GetLocalPath());
+			localPath.AddSegment(newFileItem->GetLocalFile());
+			wxFileName::Mkdir(localPath.GetPath(), 0777, wxPATH_MKDIR_FULL);
 			const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
 			for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); iter++)
-				(*iter)->RefreshLocalFile(fn.GetFullPath());
+				(*iter)->RefreshLocalFile(localPath.GetPath());
 			if (RemoveItem(newFileItem, true))
 			{
 				// Server got deleted. Unfortunately we have to start over now
@@ -985,14 +990,14 @@ void CQueueView::ProcessReply(t_EngineData* pEngineData, COperationNotification*
 		enum ResetReason reason;
 		if (pEngineData->pItem)
 		{
-			if (pEngineData->pItem->m_remove)
+			if (pEngineData->pItem->pending_remove())
 				reason = remove;
 			else
 			{
-				if (pEngineData->pItem->GetType() == QueueItemType_File && ((CFileItem*)pEngineData->pItem)->m_madeProgress)
+				if (pEngineData->pItem->GetType() == QueueItemType_File && ((CFileItem*)pEngineData->pItem)->made_progress())
 				{
 					CFileItem* pItem = (CFileItem*)pEngineData->pItem;
-					pItem->m_madeProgress = false;
+					pItem->set_made_progress(false);
 					pItem->m_onetime_action = CFileExistsNotification::resume;
 				}
 				reason = reset;
@@ -1073,12 +1078,12 @@ void CQueueView::ProcessReply(t_EngineData* pEngineData, COperationNotification*
 		// Increase error count only if item didn't make any progress. This keeps
 		// user interaction at a minimum if connection is unstable.
 
-		if (pEngineData->pItem->GetType() == QueueItemType_File && ((CFileItem*)pEngineData->pItem)->m_madeProgress &&
+		if (pEngineData->pItem->GetType() == QueueItemType_File && ((CFileItem*)pEngineData->pItem)->made_progress() &&
 			(replyCode & FZ_REPLY_WRITEFAILED) != FZ_REPLY_WRITEFAILED)
 		{
 			// Don't increase error count if there has been progress
 			CFileItem* pItem = (CFileItem*)pEngineData->pItem;
-			pItem->m_madeProgress = false;
+			pItem->set_made_progress(false);
 			pItem->m_onetime_action = CFileExistsNotification::resume;
 		}
 		else
@@ -1162,7 +1167,7 @@ void CQueueView::ProcessReply(t_EngineData* pEngineData, COperationNotification*
 	if (!m_activeMode)
 	{
 		enum ResetReason reason;
-		if (pEngineData->pItem && pEngineData->pItem->m_remove)
+		if (pEngineData->pItem && pEngineData->pItem->pending_remove())
 			reason = remove;
 		else
 			reason = reset;
@@ -1215,7 +1220,7 @@ void CQueueView::ResetEngine(t_EngineData& data, const enum ResetReason reason)
 			{
 				const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
 				for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); iter++)
-					(*iter)->RefreshLocalFile(pFileItem->GetLocalFile());
+					(*iter)->RefreshLocalFile(pFileItem->GetLocalPath().GetPath() + pFileItem->GetLocalFile());
 			}
 
 			if (pFileItem->m_edit != CEditHandler::none && reason != retry && reason != reset)
@@ -1228,7 +1233,7 @@ void CQueueView::ResetEngine(t_EngineData& data, const enum ResetReason reason)
 					pEditHandler->FinishTransfer(reason == success, pFileItem->GetRemoteFile(), pFileItem->GetRemotePath(), pServerItem->GetServer());
 				}
 				else
-					pEditHandler->FinishTransfer(reason == success, pFileItem->GetLocalFile());
+					pEditHandler->FinishTransfer(reason == success, pFileItem->GetLocalPath().GetPath() + pFileItem->GetLocalFile());
 				if (reason == success)
 					pFileItem->m_edit = CEditHandler::none;
 			}
@@ -1236,7 +1241,7 @@ void CQueueView::ResetEngine(t_EngineData& data, const enum ResetReason reason)
 			if (reason == failure)
 			{
 				pFileItem->m_onetime_action = CFileExistsNotification::unknown;
-				pFileItem->m_madeProgress = false;
+				pFileItem->set_made_progress(false);
 			}
 		}
 
@@ -1259,7 +1264,7 @@ void CQueueView::ResetEngine(t_EngineData& data, const enum ResetReason reason)
 
 		if (reason == reset)
 		{
-			if (!data.pItem->Queued())
+			if (!data.pItem->queued())
 				reinterpret_cast<CServerItem*>(data.pItem->GetTopLevelItem())->QueueImmediateFile(data.pItem);
 		}
 		else if (reason == failure)
@@ -1468,7 +1473,7 @@ void CQueueView::SendNextCommand(t_EngineData& engineData)
 			fileItem->m_statusMessage = _("Transferring");
 			RefreshItem(engineData.pItem);
 
-			int res = engineData.pEngine->Command(CFileTransferCommand(fileItem->GetLocalFile(), fileItem->GetRemotePath(),
+			int res = engineData.pEngine->Command(CFileTransferCommand(fileItem->GetLocalPath().GetPath() + fileItem->GetLocalFile(), fileItem->GetRemotePath(),
 												fileItem->GetRemoteFile(), fileItem->Download(), fileItem->m_transferSettings));
 			wxASSERT((res & FZ_REPLY_BUSY) != FZ_REPLY_BUSY);
 			if (res == FZ_REPLY_WOULDBLOCK)
@@ -1757,7 +1762,7 @@ void CQueueView::DisplayQueueSize()
 	pStatusBar->DisplayQueueSize(m_totalQueueSize, m_filesWithUnknownSize != 0);
 }
 
-bool CQueueView::QueueFolder(bool queueOnly, bool download, const wxString& localPath, const CServerPath& remotePath, const CServer& server)
+bool CQueueView::QueueFolder(bool queueOnly, bool download, const CLocalPath& localPath, const CServerPath& remotePath, const CServer& server)
 {
 	CServerItem* pServerItem = CreateServerItem(server);
 
@@ -1804,7 +1809,7 @@ void CQueueView::ProcessUploadFolderItems()
 
 	CFolderScanItem* pItem = m_queuedFolders[1].front();
 
-	if (pItem->Queued())
+	if (pItem->queued())
 		pItem->m_statusMessage = _("Scanning for files to add to queue");
 	else
 		pItem->m_statusMessage = _("Scanning for files to upload");
@@ -1829,9 +1834,9 @@ void CQueueView::OnFolderThreadComplete(wxCommandEvent& event)
 	if (pItem->m_dir_is_empty)
 	{
 		CServerItem* pServerItem = (CServerItem*)pItem->GetTopLevelItem();
-		CFileItem* fileItem = new CFolderItem(pServerItem, pItem->Queued(), pItem->m_current_remote_path, _T(""));
+		CFileItem* fileItem = new CFolderItem(pServerItem, pItem->queued(), pItem->m_current_remote_path, _T(""));
 		InsertItem(pServerItem, fileItem);
-		QueueFile_Finish(!pItem->Queued());
+		QueueFile_Finish(!pItem->queued());
 	}
 	m_queuedFolders[1].pop_front();
 
@@ -1866,7 +1871,7 @@ int CQueueView::QueueFiles(const std::list<CFolderProcessingEntry*> &entryList,
 			}
 
 			const CFolderProcessingThread::t_dirPair* entry = (const CFolderProcessingThread::t_dirPair*)*iter;
-			pFolderScanItem->m_current_local_path = entry->localPath;
+			pFolderScanItem->m_current_local_path = CLocalPath(entry->localPath);
 			pFolderScanItem->m_current_remote_path = entry->remotePath;
 			pFolderScanItem->m_dir_is_empty = true;
 			delete entry;
@@ -1875,7 +1880,7 @@ int CQueueView::QueueFiles(const std::list<CFolderProcessingEntry*> &entryList,
 		{
 			const t_newEntry* entry = (const t_newEntry*)*iter;
 
-			if (filters.FilenameFiltered(entry->name, pFolderScanItem->m_current_local_path, entry->dir, entry->size, true, entry->attributes))
+			if (filters.FilenameFiltered(entry->name, pFolderScanItem->m_current_local_path.GetPath(), entry->dir, entry->size, true, entry->attributes, &entry->time))
 			{
 				delete entry;
 				continue;
@@ -1890,7 +1895,7 @@ int CQueueView::QueueFiles(const std::list<CFolderProcessingEntry*> &entryList,
 				continue;
 			}
 
-			CFileItem* fileItem = new CFileItem(pServerItem, queueOnly, download, pFolderScanItem->m_current_local_path + CLocalFileSystem::path_separator + entry->name, entry->name, pFolderScanItem->m_current_remote_path, entry->size);
+			CFileItem* fileItem = new CFileItem(pServerItem, queueOnly, download, pFolderScanItem->m_current_local_path, entry->name, entry->name, pFolderScanItem->m_current_remote_path, entry->size);
 
 			if (download)
 				fileItem->m_transferSettings.binary = !CAutoAsciiFiles::TransferRemoteAsAscii(entry->name, pFolderScanItem->m_current_remote_path.GetType());
@@ -1915,14 +1920,14 @@ int CQueueView::QueueFiles(const std::list<CFolderProcessingEntry*> &entryList,
 void CQueueView::SaveQueue()
 {
 	// Kiosk mode doesn't save queue
-	if (COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE) == 2)
+	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
 		return;
 
 	// We have to synchronize access to queue.xml so that multiple processed don't write
 	// to the same file or one is reading while the other one writes.
 	CInterProcessMutex mutex(MUTEX_QUEUE);
 
-	wxFileName file(wxGetApp().GetSettingsDir(), _T("queue.xml"));
+	wxFileName file(COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR), _T("queue.xml"));
 	CXmlFile xml(file);
 	TiXmlElement* pDocument = xml.Load();
 	if (!pDocument)
@@ -1949,7 +1954,7 @@ void CQueueView::LoadQueue()
 	// to the same file or one is reading while the other one writes.
 	CInterProcessMutex mutex(MUTEX_QUEUE);
 
-	wxFileName file(wxGetApp().GetSettingsDir(), _T("queue.xml"));
+	wxFileName file(COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR), _T("queue.xml"));
 	CXmlFile xml(file);
 	TiXmlElement* pDocument = xml.Load();
 	if (!pDocument)
@@ -1968,7 +1973,7 @@ void CQueueView::LoadQueue()
 
 	pDocument->RemoveChild(pQueue);
 
-	if (COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE) == 2)
+	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
 		return;
 
 	wxString error;
@@ -1991,6 +1996,9 @@ void CQueueView::ImportQueue(TiXmlElement* pElement, bool updateSelections)
 			m_insertionCount = 0;
 			CServerItem *pServerItem = CreateServerItem(server);
 
+			CLocalPath previousLocalPath;
+			CServerPath previousRemotePath;
+
 			for (TiXmlElement* pFile = pServer->FirstChildElement("File"); pFile; pFile = pFile->NextSiblingElement("File"))
 			{
 				wxString localFile = GetTextElement(pFile, "LocalFile");
@@ -2007,7 +2015,20 @@ void CQueueView::ImportQueue(TiXmlElement* pElement, bool updateSelections)
 				if (localFile != _T("") && remoteFile != _T("") && remotePath.SetSafePath(safeRemotePath) &&
 					size >= -1 && priority < PRIORITY_COUNT)
 				{
-					CFileItem* fileItem = new CFileItem(pServerItem, true, download, localFile, remoteFile, remotePath, size);
+					wxString localFileName;
+					CLocalPath localPath(localFile, &localFileName);
+
+					if (localFileName.empty())
+						continue;
+
+					// CServerPath and wxString are reference counted.
+					// Save some memory here by re-using the old copy
+					if (localPath != previousLocalPath)
+						previousLocalPath = localPath;
+					if (previousRemotePath != remotePath)
+						previousRemotePath = remotePath;
+
+					CFileItem* fileItem = new CFileItem(pServerItem, true, download, localPath, localFileName, remoteFile, previousRemotePath, size);
 					fileItem->m_transferSettings.binary = binary;
 					fileItem->SetPriorityRaw((enum QueuePriority)priority);
 					fileItem->m_errorCount = errorCount;
@@ -2027,7 +2048,7 @@ void CQueueView::ImportQueue(TiXmlElement* pElement, bool updateSelections)
 					wxString localFile = GetTextElement(pFolder, "LocalFile");
 					if (localFile == _T(""))
 						continue;
-					folderItem = new CFolderItem(pServerItem, true, localFile);
+					folderItem = new CFolderItem(pServerItem, true, CLocalPath(localFile));
 				}
 				else
 				{
@@ -2115,12 +2136,7 @@ void CQueueView::OnContextMenu(wxContextMenuEvent& event)
 	if (!pMenu)
 		return;
 
-#ifndef __WXMSW__
-	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
-	const bool has_selection = GetSelectedItemCount() != 0;
-#else
-	const bool has_selection = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) != -1;
-#endif
+	bool has_selection = HasSelection();
 
 	pMenu->Check(XRCID("ID_PROCESSQUEUE"), IsActive() ? true : false);
 	pMenu->Check(XRCID("ID_ACTIONAFTER_DISABLE"), IsActionAfter(ActionAfterState_Disabled));
@@ -2313,7 +2329,7 @@ void CQueueView::OnRemoveSelected(wxCommandEvent& event)
 			CFileItem* pFile = (CFileItem*)pItem;
 			if (pFile->IsActive())
 			{
-				pFile->m_remove = true;
+				pFile->set_pending_remove(true);
 				StopItem(pFile);
 				continue;
 			}
@@ -2349,7 +2365,7 @@ bool CQueueView::StopItem(CFileItem* item)
 	if (item->m_pEngineData->state == t_EngineData::waitprimary)
 	{
 		enum ResetReason reason;
-		if (item->m_pEngineData->pItem && item->m_pEngineData->pItem->m_remove)
+		if (item->m_pEngineData->pItem && item->m_pEngineData->pItem->pending_remove())
 			reason = remove;
 		else
 			reason = reset;
@@ -2389,7 +2405,7 @@ bool CQueueView::StopItem(CServerItem* pServerItem)
 			CFileItem* pFile = (CFileItem*)pItem;
 			if (pFile->IsActive())
 			{
-				pFile->m_remove = true;
+				pFile->set_pending_remove(true);
 				StopItem(pFile);
 				continue;
 			}
@@ -2421,7 +2437,7 @@ void CQueueView::OnFolderThreadFiles(wxCommandEvent& event)
 
 	std::list<CFolderProcessingEntry*> entryList;
 	m_pFolderProcessingThread->GetFiles(entryList);
-	int added = QueueFiles(entryList, pItem->Queued(), false, (CServerItem*)pItem->GetTopLevelItem(), pItem->m_defaultFileExistsAction);
+	int added = QueueFiles(entryList, pItem->queued(), false, (CServerItem*)pItem->GetTopLevelItem(), pItem->m_defaultFileExistsAction);
 	m_pFolderProcessingThread->CheckFinished();
 
 	pItem->m_count += added;
@@ -2438,13 +2454,7 @@ void CQueueView::SetDefaultFileExistsAction(enum CFileExistsNotification::Overwr
 
 void CQueueView::OnSetDefaultFileExistsAction(wxCommandEvent &event)
 {
-#ifndef __WXMSW__
-	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
-	const bool has_selection = GetSelectedItemCount() != 0;
-#else
-	const bool has_selection = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) != -1;
-#endif
-	if (!has_selection)
+	if (!HasSelection())
 		return;
 
 	CDefaultFileExistsDlg dlg;
@@ -3254,7 +3264,7 @@ bool CQueueView::IsOtherEngineConnected(t_EngineData* pEngineData)
 
 void CQueueView::OnChar(wxKeyEvent& event)
 {
-	if (event.GetKeyCode() == WXK_DELETE)
+	if (event.GetKeyCode() == WXK_DELETE || event.GetKeyCode() == WXK_NUMPAD_DELETE)
 	{
 		wxCommandEvent cmdEvt;
 		OnRemoveSelected(cmdEvt);
@@ -3313,9 +3323,9 @@ void CQueueView::RenameFileInTransfer(CFileZillaEngine *pEngine, const wxString&
 	CFileItem* pFile = (CFileItem*)pEngineData->pItem;
 	if (local)
 	{
-		wxFileName fn(pFile->GetLocalFile());
+		wxFileName fn(pFile->GetLocalPath().GetPath(), pFile->GetLocalFile());
 		fn.SetFullName(newName);
-		pFile->SetLocalFile(fn.GetFullPath());
+		pFile->SetLocalFile(fn.GetFullName());
 	}
 	else
 		pFile->SetRemoteFile(newName);
