@@ -1,11 +1,21 @@
 #include <filezilla.h>
 #include "listctrlex.h"
+#include "filezillaapp.h"
 #include <wx/renderer.h>
 #include <wx/tokenzr.h>
 #include "Options.h"
 #include "dialogex.h"
 #ifdef __WXMSW__
 #include "commctrl.h"
+
+#ifndef HDF_SORTUP
+#define HDF_SORTUP              0x0400
+#endif
+#ifndef HDF_SORTDOWN
+#define HDF_SORTDOWN            0x0200
+#endif
+#else
+#include "themeprovider.h"
 #endif
 
 DECLARE_EVENT_TYPE(fzEVT_POSTSCROLL, -1)
@@ -20,8 +30,13 @@ EVT_LIST_ITEM_SELECTED(wxID_ANY, wxListCtrlEx::OnSelectionChanged)
 EVT_KEY_DOWN(wxListCtrlEx::OnKeyDown)
 EVT_LIST_BEGIN_LABEL_EDIT(wxID_ANY, wxListCtrlEx::OnBeginLabelEdit)
 EVT_LIST_END_LABEL_EDIT(wxID_ANY, wxListCtrlEx::OnEndLabelEdit)
+#ifndef __WXMSW__
+EVT_LIST_COL_DRAGGING(wxID_ANY, wxListCtrlEx::OnColumnDragging)
+#endif
 END_EVENT_TABLE()
 
+#define MIN_COLUMN_WIDTH 12
+
 wxListCtrlEx::wxListCtrlEx(wxWindow *parent,
 						   wxWindowID id,
 						   const wxPoint& pos,
@@ -31,17 +46,27 @@ wxListCtrlEx::wxListCtrlEx(wxWindow *parent,
 						   const wxString& name)
 						   : wxListCtrl(parent, id, pos, size, style, validator, name)
 {
+#ifdef __WXMSW__
+	m_pHeaderImageList = 0;
+#endif
+	m_header_icon_index.down = m_header_icon_index.up = -1;
+
 	m_pVisibleColumnMapping = 0;
 	m_prefixSearch_enabled = false;
 
 #ifndef __WXMSW__
 	m_editing = false;
+#else
+	m_columnDragging = false;
 #endif
 	m_blockedLabelEditing = false;
 }
 
 wxListCtrlEx::~wxListCtrlEx()
 {
+#ifdef __WXMSW__
+	delete m_pHeaderImageList;
+#endif
 	delete [] m_pVisibleColumnMapping;
 }
 
@@ -528,6 +553,8 @@ bool wxListCtrlEx::ReadColumnWidths(unsigned int optionId)
 			delete [] newWidths;
 			return false;
 		}
+		else if (newWidths[i] < MIN_COLUMN_WIDTH)
+			newWidths[i] = MIN_COLUMN_WIDTH;
 	}
 
 	for (unsigned int i = 0; i < count; i++)
@@ -614,7 +641,7 @@ void wxListCtrlEx::MoveColumn(unsigned int col, unsigned int before)
 				m_pVisibleColumnMapping[j - 1] = m_pVisibleColumnMapping[j];
 			info.width = GetColumnWidth(i);
 
-			icon = GetHeaderIconIndex(i);
+			icon = GetHeaderSortIconIndex(i);
 			DeleteColumn(i);
 
 			break;
@@ -636,7 +663,7 @@ void wxListCtrlEx::MoveColumn(unsigned int col, unsigned int before)
 
 		InsertColumn(pos, info.name, info.align, info.width);
 
-		SetHeaderIconIndex(pos, icon);
+		SetHeaderSortIconIndex(pos, icon);
 	}
 	m_columnInfo[col].order = before;
 }
@@ -808,7 +835,7 @@ int wxListCtrlEx::GetColumnVisibleIndex(int col)
 	return -1;
 }
 
-int wxListCtrlEx::GetHeaderIconIndex(int col)
+int wxListCtrlEx::GetHeaderSortIconIndex(int col)
 {
 	if (col < 0 || col >= GetColumnCount())
 		return -1;
@@ -834,7 +861,69 @@ int wxListCtrlEx::GetHeaderIconIndex(int col)
 #endif
 }
 
-void wxListCtrlEx::SetHeaderIconIndex(int col, int icon)
+void wxListCtrlEx::InitHeaderSortImageList()
+{
+#ifdef __WXMSW__
+	if (wxPlatformInfo::Get().GetOSMajorVersion() >= 6)
+		return;
+
+	// Initialize imagelist for list header
+	m_pHeaderImageList = new wxImageListEx(8, 8, true, 3);
+
+	wxBitmap bmp;
+
+	bmp.LoadFile(wxGetApp().GetResourceDir() + _T("up.png"), wxBITMAP_TYPE_PNG);
+	m_pHeaderImageList->Add(bmp);
+	bmp.LoadFile(wxGetApp().GetResourceDir() + _T("down.png"), wxBITMAP_TYPE_PNG);
+	m_pHeaderImageList->Add(bmp);
+
+	HWND hWnd = (HWND)GetHandle();
+	if (!hWnd)
+	{
+		delete m_pHeaderImageList;
+		m_pHeaderImageList = 0;
+		return;
+	}
+
+	HWND header = (HWND)SendMessage(hWnd, LVM_GETHEADER, 0, 0);
+	if (!header)
+	{
+		delete m_pHeaderImageList;
+		m_pHeaderImageList = 0;
+		return;
+	}
+
+	TCHAR buffer[1000] = {0};
+	HDITEM item;
+	item.mask = HDI_TEXT;
+	item.pszText = buffer;
+	item.cchTextMax = 999;
+	SendMessage(header, HDM_GETITEM, 0, (LPARAM)&item);
+
+	SendMessage(header, HDM_SETIMAGELIST, 0, (LPARAM)m_pHeaderImageList->GetHandle());
+
+	m_header_icon_index.up = 0;
+	m_header_icon_index.down = 1;
+#else
+
+	wxColour colour = wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE);
+
+	wxString lightness;
+	if (colour.Red() + colour.Green() + colour.Blue() > 3 * 128)
+		lightness = _T("DARK");
+	else
+		lightness = _T("LIGHT");
+
+	wxBitmap bmp;
+
+	bmp = wxArtProvider::GetBitmap(_T("ART_SORT_UP_") + lightness, wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall));
+	m_header_icon_index.up = m_pImageList->Add(bmp);
+	bmp = wxArtProvider::GetBitmap(_T("ART_SORT_DOWN_") + lightness, wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall));
+	m_header_icon_index.down = m_pImageList->Add(bmp);
+#endif
+}
+
+void wxListCtrlEx::SetHeaderSortIconIndex(int col, int icon)
 {
 	if (col < 0 || col >= GetColumnCount())
 		return;
@@ -844,25 +933,44 @@ void wxListCtrlEx::SetHeaderIconIndex(int col, int icon)
 	HWND header = (HWND)SendMessage(hWnd, LVM_GETHEADER, 0, 0);
 
 	wxChar buffer[100];
-	HDITEM item;
+	HDITEM item = {0};
 	item.mask = HDI_TEXT | HDI_FORMAT;
 	item.pszText = buffer;
 	item.cchTextMax = 99;
 	SendMessage(header, HDM_GETITEM, col, (LPARAM)&item);
-	item.mask |= HDI_IMAGE;
 	if (icon != -1)
 	{
-		item.fmt |= HDF_IMAGE | HDF_BITMAP_ON_RIGHT;
-		item.iImage = icon;
+		if (wxPlatformInfo::Get().GetOSMajorVersion() >= 6)
+		{
+			item.fmt &= ~(HDF_IMAGE | HDF_BITMAP_ON_RIGHT | HDF_SORTUP | HDF_SORTDOWN);
+			item.iImage = -1;
+			if (icon)
+				item.fmt |= HDF_SORTDOWN;
+			else
+				item.fmt |= HDF_SORTUP;
+		}
+		else
+		{
+			item.fmt &= ~(HDF_SORTUP | HDF_SORTDOWN);
+			item.fmt |= HDF_IMAGE | HDF_BITMAP_ON_RIGHT;
+			item.iImage = icon ? m_header_icon_index.down : m_header_icon_index.up;
+			item.mask |= HDI_IMAGE;
+		}
 	}
 	else
-		item.fmt &= ~(HDF_IMAGE | HDF_BITMAP_ON_RIGHT);
+	{
+		item.fmt &= ~(HDF_IMAGE | HDF_BITMAP_ON_RIGHT | HDF_SORTUP | HDF_SORTDOWN);
+		item.iImage = -1;
+	}
 	SendMessage(header, HDM_SETITEM, col, (LPARAM)&item);
 #else
 	wxListItem item;
 	if (!GetColumn(col, item))
 		return;
 
+	if( icon != -1 )
+		icon = icon ? m_header_icon_index.down : m_header_icon_index.up;
+
 	item.SetImage(icon);
 	SetColumn(col, item);
 #endif
@@ -989,3 +1097,59 @@ CLabelEditBlocker::~CLabelEditBlocker()
 {
 	m_listCtrl.SetLabelEditBlock(false);
 }
+
+void wxListCtrlEx::OnColumnDragging(wxListEvent& event)
+{
+	if (event.GetItem().GetWidth() < MIN_COLUMN_WIDTH)
+		event.Veto();
+}
+
+#ifdef __WXMSW__
+bool wxListCtrlEx::MSWOnNotify(int idCtrl, WXLPARAM lParam, WXLPARAM *result)
+{
+	// MSW doesn't generate HDN_TRACK on all header styles, so handle it
+	// ourselves using HDN_ITEMCHANGING.
+	NMHDR *nmhdr = (NMHDR *)lParam;
+	HWND hwndHdr = ListView_GetHeader((HWND)GetHandle());
+
+    if (nmhdr->hwndFrom != hwndHdr)
+		return wxListCtrl::MSWOnNotify(idCtrl, lParam, result);
+
+	HD_NOTIFY *nmHDR = (HD_NOTIFY *)nmhdr;
+
+	switch ( nmhdr->code )
+	{
+		// See comment in src/msw/listctrl.cpp of wx why both A and W are needed
+	case HDN_BEGINTRACKA:
+	case HDN_BEGINTRACKW:
+		m_columnDragging = true;
+		break;
+	case HDN_ENDTRACKA:
+	case HDN_ENDTRACKW:
+		m_columnDragging = true;
+		break;
+	case HDN_ITEMCHANGINGA:
+	case HDN_ITEMCHANGINGW:
+		if (m_columnDragging)
+		{
+			if (nmHDR->pitem->mask & HDI_WIDTH && nmHDR->pitem->cxy < 10)
+				*result = 1;
+			else
+				*result = 0;
+		}
+		return true;
+	}
+
+	return wxListCtrl::MSWOnNotify(idCtrl, lParam, result);
+}
+#endif
+
+bool wxListCtrlEx::HasSelection() const
+{
+#ifndef __WXMSW__
+	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
+	return GetSelectedItemCount() != 0;
+#else
+	return GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) != -1;
+#endif
+}
