@@ -357,7 +357,16 @@ void CManualTransfer::OnOK(wxCommandEvent& event)
 	else
 		COptions::Get()->SetOption(OPTION_ASCIIBINARY, 0);
 
-	m_pQueueView->QueueFile(!start, download, local_file, remote_file, path, *m_pServer, -1);
+	wxString name;
+	CLocalPath localPath(local_file, &name);
+
+	if (name.empty())
+	{
+		wxMessageBox(_("Local file is not a valid filename."), _("Manual transfer"), wxICON_EXCLAMATION);
+		return;
+	}
+
+	m_pQueueView->QueueFile(!start, download, localPath, name, remote_file, path, *m_pServer, -1);
 
 	// Restore old data type
 	COptions::Get()->SetOption(OPTION_ASCIIBINARY, old_data_type);
@@ -430,7 +439,7 @@ bool CManualTransfer::VerifyServer()
 		return false;
 	}
 
-	if (COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE) != 0 &&
+	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) != 0 &&
 		(logon_type == ACCOUNT || logon_type == NORMAL))
 	{
 		XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->SetFocus();
