@@ -4,6 +4,7 @@
 #include "queueview_failed.h"
 #include "queueview_successful.h"
 #include "sizeformatting.h"
+#include "themeprovider.h"
 
 CQueueItem::CQueueItem()
 {
@@ -278,22 +279,25 @@ int CQueueItem::GetItemIndex() const
 	return index + pParent->GetItemIndex();
 }
 
-CFileItem::CFileItem(CServerItem* parent, bool queued, bool download, const wxString& localFile,
+CFileItem::CFileItem(CServerItem* parent, bool queued, bool download,
+					 const CLocalPath& localPath, const wxString& localFile,
 					 const wxString& remoteFile, const CServerPath& remotePath, wxLongLong size)
-	: m_localFile(localFile), m_remoteFile(remoteFile), m_remotePath(remotePath), m_size(size)
+	: m_localFile(localFile), m_remoteFile(remoteFile)
+	, m_localPath(localPath), m_remotePath(remotePath)
+	, m_size(size)
 {
 	m_parent = parent;
 	m_priority = priority_normal;
 
-	m_download = download;
-	m_queued = queued;
-	m_active = false;
+	flags = 0;
+	if (download)
+		flags |= flag_download;
+	if (queued)
+		flags |= flag_queued;
 	m_errorCount = 0;
-	m_remove = false;
 	m_pEngineData = 0;
 	m_defaultFileExistsAction = CFileExistsNotification::unknown;
 	m_edit = CEditHandler::none;
-	m_madeProgress = false;
 	m_onetime_action = CFileExistsNotification::unknown;
 }
 
@@ -323,16 +327,17 @@ enum QueuePriority CFileItem::GetPriority() const
 
 void CFileItem::SetActive(const bool active)
 {
-	if (active && !m_active)
+	if (active && !IsActive())
 	{
 		AddChild(new CStatusItem);
+		flags |= flag_active;
 	}
-	else if (!active && m_active)
+	else if (!active && IsActive())
 	{
 		CQueueItem* pItem = GetChild(0, false);
 		RemoveChild(pItem);
+		flags &= ~flag_active;
 	}
-	m_active = active;
 }
 
 void CFileItem::SaveItem(TiXmlElement* pElement) const
@@ -342,10 +347,10 @@ void CFileItem::SaveItem(TiXmlElement* pElement) const
 
 	TiXmlElement *file = pElement->LinkEndChild(new TiXmlElement("File"))->ToElement();
 
-	AddTextElement(file, "LocalFile", m_localFile);
+	AddTextElement(file, "LocalFile", m_localPath.GetPath() + m_localFile);
 	AddTextElement(file, "RemoteFile", m_remoteFile);
 	AddTextElement(file, "RemotePath", m_remotePath.GetSafePath());
-	AddTextElementRaw(file, "Download", m_download ? "1" : "0");
+	AddTextElementRaw(file, "Download", Download() ? "1" : "0");
 	if (m_size != -1)
 		AddTextElement(file, "Size", m_size.ToString());
 	if (m_errorCount)
@@ -362,7 +367,7 @@ bool CFileItem::TryRemoveAll()
 	if (!IsActive())
 		return true;
 
-	m_remove = true;
+	set_pending_remove(true);
 	return false;
 }
 
@@ -378,13 +383,13 @@ void CFileItem::SetRemoteFile(const wxString &file)
 	m_remoteFile = file;
 }
 
-CFolderItem::CFolderItem(CServerItem* parent, bool queued, const wxString& localFile)
-	: CFileItem(parent, queued, true, localFile, _T(""), CServerPath(), -1)
+CFolderItem::CFolderItem(CServerItem* parent, bool queued, const CLocalPath& localPath)
+	: CFileItem(parent, queued, true, localPath, _T(""), _T(""), CServerPath(), -1)
 {
 }
 
 CFolderItem::CFolderItem(CServerItem* parent, bool queued, const CServerPath& remotePath, const wxString& remoteFile)
-	: CFileItem(parent, queued, false, _T(""), remoteFile, remotePath, -1)
+	: CFileItem(parent, queued, false, CLocalPath(), _T(""), remoteFile, remotePath, -1)
 {
 }
 
@@ -392,21 +397,24 @@ void CFolderItem::SaveItem(TiXmlElement* pElement) const
 {
 	TiXmlElement *file = new TiXmlElement("Folder");
 
-	if (m_download)
+	if (Download())
 		AddTextElement(file, "LocalFile", m_localFile);
 	else
 	{
 		AddTextElement(file, "RemoteFile", m_remoteFile);
 		AddTextElement(file, "RemotePath", m_remotePath.GetSafePath());
 	}
-	AddTextElementRaw(file, "Download", m_download ? "1" : "0");
+	AddTextElementRaw(file, "Download", Download() ? "1" : "0");
 
 	pElement->LinkEndChild(file);
 }
 
 void CFolderItem::SetActive(const bool active)
 {
-	m_active = active;
+	if (active)
+		flags |= flag_active;
+	else
+		flags &= ~flag_active;
 }
 
 CServerItem::CServerItem(const CServer& server)
@@ -442,12 +450,12 @@ void CServerItem::AddFileItemToList(CFileItem* pItem)
 	if (!pItem)
 		return;
 
-	m_fileList[pItem->m_queued ? 0 : 1][pItem->GetPriority()].push_back(pItem);
+	m_fileList[pItem->queued() ? 0 : 1][pItem->GetPriority()].push_back(pItem);
 }
 
 void CServerItem::RemoveFileItemFromList(CFileItem* pItem)
 {
-	std::list<CFileItem*>& fileList = m_fileList[pItem->m_queued ? 0 : 1][pItem->GetPriority()];
+	std::list<CFileItem*>& fileList = m_fileList[pItem->queued() ? 0 : 1][pItem->GetPriority()];
 	for (std::list<CFileItem*>::iterator iter = fileList.begin(); iter != fileList.end(); iter++)
 	{
 		if (*iter == pItem)
@@ -557,12 +565,12 @@ void CServerItem::QueueImmediateFiles()
 		for (std::list<CFileItem*>::reverse_iterator iter = fileList.rbegin(); iter != fileList.rend(); iter++)
 		{
 			CFileItem* item = *iter;
-			wxASSERT(!item->m_queued);
+			wxASSERT(!item->queued());
 			if (item->IsActive())
 				activeList.push_front(item);
 			else
 			{
-				item->m_queued = true;
+				item->set_queued(true);
 				m_fileList[0][i].push_front(item);
 			}
 		}
@@ -572,7 +580,7 @@ void CServerItem::QueueImmediateFiles()
 
 void CServerItem::QueueImmediateFile(CFileItem* pItem)
 {
-	if (pItem->m_queued)
+	if (pItem->queued())
 		return;
 
 	std::list<CFileItem*>& fileList = m_fileList[1][pItem->GetPriority()];
@@ -581,7 +589,7 @@ void CServerItem::QueueImmediateFile(CFileItem* pItem)
 		if (*iter != pItem)
 			continue;
 
-		pItem->m_queued = true;
+		pItem->set_queued(true);
 		fileList.erase(iter);
 		m_fileList[0][pItem->GetPriority()].push_front(pItem);
 		return;
@@ -703,7 +711,7 @@ void CServerItem::SetPriority(enum QueuePriority priority)
 
 void CServerItem::SetChildPriority(CFileItem* pItem, enum QueuePriority oldPriority, enum QueuePriority newPriority)
 {
-	int i = pItem->Queued() ? 0 : 1;
+	int i = pItem->queued() ? 0 : 1;
 
 	for (std::list<CFileItem*>::iterator iter = m_fileList[i][oldPriority].begin(); iter != m_fileList[i][oldPriority].end(); iter++)
 	{
@@ -718,7 +726,7 @@ void CServerItem::SetChildPriority(CFileItem* pItem, enum QueuePriority oldPrior
 	wxFAIL;
 }
 
-CFolderScanItem::CFolderScanItem(CServerItem* parent, bool queued, bool download, const wxString& localPath, const CServerPath& remotePath)
+CFolderScanItem::CFolderScanItem(CServerItem* parent, bool queued, bool download, const CLocalPath& localPath, const CServerPath& remotePath)
 {
 	m_parent = parent;
 
@@ -772,12 +780,13 @@ CQueueViewBase::CQueueViewBase(CQueue* parent, int index, const wxString& title)
 	m_folderScanCountChanged = false;
 
 	// Create and assign the image list for the queue
-	wxImageList* pImageList = new wxImageList(16, 16);
+	wxSize s = CThemeProvider::GetIconSize(iconSizeSmall);
+	wxImageList* pImageList = new wxImageList(s.x, s.y);
 
-	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_SERVER"),  wxART_OTHER, wxSize(16, 16)));
-	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_FILE"),  wxART_OTHER, wxSize(16, 16)));
-	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_FOLDERCLOSED"),  wxART_OTHER, wxSize(16, 16)));
-	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_FOLDER"),  wxART_OTHER, wxSize(16, 16)));
+	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_SERVER"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall)));
+	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_FILE"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall)));
+	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_FOLDERCLOSED"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall)));
+	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_FOLDER"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall)));
 
 	AssignImageList(pImageList, wxIMAGE_LIST_SMALL);
 
@@ -855,15 +864,15 @@ wxString CQueueViewBase::OnGetItemText(long item, long column) const
 			switch (column)
 			{
 			case 0:
-				return pFileItem->GetIndent() + pFileItem->GetLocalFile();
+				return pFileItem->GetIndent() + pFileItem->GetLocalPath().GetPath() + pFileItem->GetLocalFile();
 			case 1:
 				if (pFileItem->Download())
-					if (pFileItem->Queued())
+					if (pFileItem->queued())
 						return _T("<--");
 					else
 						return _T("<<--");
 				else
-					if (pFileItem->Queued())
+					if (pFileItem->queued())
 						return _T("-->");
 					else
 						return _T("-->>");
@@ -907,15 +916,15 @@ wxString CQueueViewBase::OnGetItemText(long item, long column) const
 			switch (column)
 			{
 			case 0:
-				return _T("  ") + pFolderItem->GetLocalPath();
+				return _T("  ") + pFolderItem->GetLocalPath().GetPath();
 			case 1:
 				if (pFolderItem->Download())
-					if (pFolderItem->Queued())
+					if (pFolderItem->queued())
 						return _T("<--");
 					else
 						return _T("<<--");
 				else
-					if (pFolderItem->Queued())
+					if (pFolderItem->queued())
 						return _T("-->");
 					else
 						return _T("-->>");
@@ -936,16 +945,16 @@ wxString CQueueViewBase::OnGetItemText(long item, long column) const
 			{
 			case 0:
 				if (pFolderItem->Download())
-					return pFolderItem->GetIndent() + pFolderItem->GetLocalFile();
+					return pFolderItem->GetIndent() + pFolderItem->GetLocalPath().GetPath() + pFolderItem->GetLocalFile();
 				break;
 			case 1:
 				if (pFolderItem->Download())
-					if (pFolderItem->Queued())
+					if (pFolderItem->queued())
 						return _T("<--");
 					else
 						return _T("<<--");
 				else
-					if (pFolderItem->Queued())
+					if (pFolderItem->queued())
 						return _T("-->");
 					else
 						return _T("-->>");
