@@ -84,13 +84,11 @@ void CRecursiveOperation::StartRecursiveOperation(enum OperationMode mode, const
 	NextOperation();
 }
 
-void CRecursiveOperation::AddDirectoryToVisit(const CServerPath& path, const wxString& subdir, const wxString& localDir /*=_T("")*/, bool is_link /*=false*/)
+void CRecursiveOperation::AddDirectoryToVisit(const CServerPath& path, const wxString& subdir, const CLocalPath& localDir /*=CLocalPath()*/, bool is_link /*=false*/)
 {
 	CNewDir dirToVisit;
 
 	dirToVisit.localDir = localDir;
-	if (localDir != _T("") && localDir.Last() != CLocalFileSystem::path_separator)
-		dirToVisit.localDir += CLocalFileSystem::path_separator;
 	dirToVisit.parent = path;
 	dirToVisit.subdir = subdir;
 	dirToVisit.link = is_link ? 2 : 0;
@@ -222,13 +220,12 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 	{
 		if (m_operationMode == recursive_download)
 		{
-			wxFileName fn(dir.localDir, _T(""));
-			wxFileName::Mkdir(fn.GetPath(), 0777, wxPATH_MKDIR_FULL);
-			m_pState->RefreshLocalFile(fn.GetFullPath());
+			wxFileName::Mkdir(dir.localDir.GetPath(), 0777, wxPATH_MKDIR_FULL);
+			m_pState->RefreshLocalFile(dir.localDir.GetPath());
 		}
 		else if (m_operationMode == recursive_addtoqueue)
 		{
-			m_pQueue->QueueFile(true, true, dir.localDir, _T(""), CServerPath(), *pServer, -1);
+			m_pQueue->QueueFile(true, true, dir.localDir, _T(""), _T(""), CServerPath(), *pServer, -1);
 			m_pQueue->QueueFile_Finish(false);
 		}
 	}
@@ -253,10 +250,10 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 			if (entry.name != dir.restrict)
 				continue;
 		}
-		else if (filter.FilenameFiltered(m_filters, entry.name, path, entry.dir, entry.size, false, 0))
+		else if (filter.FilenameFiltered(m_filters, entry.name, path, entry.is_dir(), entry.size, false, 0, entry.has_date() ? &entry.time : 0))
 			continue;
 
-		if (entry.dir && (!entry.link || m_operationMode != recursive_delete))
+		if (entry.is_dir() && (!entry.is_link() || m_operationMode != recursive_delete))
 		{
 			if (dir.recurse)
 			{
@@ -266,11 +263,9 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 				dirToVisit.localDir = dir.localDir;
 				dirToVisit.start_dir = dir.start_dir;
 				
-				if (m_operationMode != recursive_addtoqueue_flatten && m_operationMode != recursive_download_flatten)
-				{
-					dirToVisit.localDir += CQueueView::ReplaceInvalidCharacters(entry.name) + CLocalFileSystem::path_separator;
-				}
-				if (entry.link)
+				if (m_operationMode == recursive_download || m_operationMode == recursive_addtoqueue)
+					dirToVisit.localDir.AddSegment(CQueueView::ReplaceInvalidCharacters(entry.name));
+				if (entry.is_link())
 				{
 					dirToVisit.link = 1;
 					dirToVisit.recurse = false;
@@ -282,13 +277,20 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 		{
 			switch (m_operationMode)
 			{
-			case recursive_addtoqueue:
 			case recursive_download:
-			case recursive_addtoqueue_flatten:
 			case recursive_download_flatten:
 				{
 					m_pQueue->QueueFile(m_operationMode == recursive_addtoqueue, true,
-						dir.localDir + CQueueView::ReplaceInvalidCharacters(entry.name),
+						dir.localDir, CQueueView::ReplaceInvalidCharacters(entry.name),
+						entry.name, pDirectoryListing->path, *pServer, entry.size);
+					added = true;
+				}
+				break;
+			case recursive_addtoqueue:
+			case recursive_addtoqueue_flatten:
+				{
+					m_pQueue->QueueFile(true, true,
+						dir.localDir, CQueueView::ReplaceInvalidCharacters(entry.name),
 						entry.name, pDirectoryListing->path, *pServer, entry.size);
 					added = true;
 				}
@@ -305,12 +307,12 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 		{
 			const int applyType = m_pChmodDlg->GetApplyType();
 			if (!applyType ||
-				(!entry.dir && applyType == 1) ||
-				(entry.dir && applyType == 2))
+				(!entry.is_dir() && applyType == 1) ||
+				(entry.is_dir() && applyType == 2))
 			{
 				char permissions[9];
 				bool res = m_pChmodDlg->ConvertPermissions(entry.permissions, permissions);
-				wxString newPerms = m_pChmodDlg->GetPermissions(res ? permissions : 0, entry.dir);
+				wxString newPerms = m_pChmodDlg->GetPermissions(res ? permissions : 0, entry.is_dir());
 				m_pState->m_pCommandQueue->ProcessCommand(new CChmodCommand(pDirectoryListing->path, entry.name, newPerms));
 			}
 		}
@@ -418,21 +420,15 @@ void CRecursiveOperation::LinkIsNotDir()
 		NextOperation();
 		return;
 	}
-
-	wxString local_file = dir.localDir;
-	if (m_operationMode == recursive_addtoqueue_flatten || m_operationMode == recursive_download_flatten)
+	else if (m_operationMode != recursive_list )
 	{
-		if (local_file.Last() != CLocalFileSystem::path_separator)
-			local_file += CLocalFileSystem::path_separator;
-		local_file += dir.subdir;
+		CLocalPath localPath = dir.localDir;
+		wxString localFile = dir.subdir;
+		if (m_operationMode != recursive_addtoqueue_flatten && m_operationMode != recursive_download_flatten)
+			localPath.MakeParent();
+		m_pQueue->QueueFile(m_operationMode == recursive_addtoqueue || m_operationMode == recursive_addtoqueue_flatten, true, localPath, localFile, dir.subdir, dir.parent, *pServer, -1);
+		m_pQueue->QueueFile_Finish(m_operationMode != recursive_addtoqueue);
 	}
-	else
-	{
-		if (local_file.Last() == CLocalFileSystem::path_separator)
-			local_file.RemoveLast();
-	}
-	m_pQueue->QueueFile(m_operationMode == recursive_addtoqueue || m_operationMode == recursive_addtoqueue_flatten, true, local_file, dir.subdir, dir.parent, *pServer, -1);
-	m_pQueue->QueueFile_Finish(m_operationMode != recursive_addtoqueue);
 
 	NextOperation();
 }
