@@ -6,6 +6,7 @@
 #include "ipcmutex.h"
 #include "wrapengine.h"
 #include "conditionaldialog.h"
+#include "themeprovider.h"
 #include "window_state_manager.h"
 #include <wx/dnd.h>
 #ifdef __WXMSW__
@@ -323,12 +324,14 @@ bool CSiteManager::Create(wxWindow* parent, std::vector<_connected_site> *connec
 	wxTreeCtrl *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrl);
 	if (!pTree)
 		return false;
-	wxImageList* pImageList = new wxImageList(16, 16);
 
-	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_FOLDERCLOSED"),  wxART_OTHER, wxSize(16, 16)));
-	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_FOLDER"),  wxART_OTHER, wxSize(16, 16)));
-	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_SERVER"),  wxART_OTHER, wxSize(16, 16)));
-	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_BOOKMARK"),  wxART_OTHER, wxSize(16, 16)));
+	wxSize s = CThemeProvider::GetIconSize(iconSizeSmall);
+	wxImageList* pImageList = new wxImageList(s.x, s.y);
+
+	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_FOLDERCLOSED"), wxART_OTHER, s));
+	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_FOLDER"), wxART_OTHER, s));
+	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_SERVER"), wxART_OTHER, s));
+	pImageList->Add(wxArtProvider::GetBitmap(_T("ART_BOOKMARK"), wxART_OTHER, s));
 
 	pTree->AssignImageList(pImageList);
 
@@ -398,6 +401,9 @@ bool CSiteManager::Create(wxWindow* parent, std::vector<_connected_site> *connec
 	XRCCTRL(*this, "ID_TRANSFERMODE_ACTIVE", wxRadioButton)->Update();
 	XRCCTRL(*this, "ID_TRANSFERMODE_PASSIVE", wxRadioButton)->Update();
 
+	wxTreeItemId item = pTree->GetSelection();
+	if (!item.IsOk())
+		pTree->SelectItem(m_ownSites);
 	SetCtrlState();
 
 	m_pWindowStateManager = new CWindowStateManager(this);
@@ -406,12 +412,14 @@ bool CSiteManager::Create(wxWindow* parent, std::vector<_connected_site> *connec
 	pTree->SetDropTarget(new CSiteManagerDropTarget(this));
 
 #ifdef __WXGTK__
-	CSiteManagerItemData* data = 0;
-	wxTreeItemId item = pTree->GetSelection();
-	if (item.IsOk())
-		data = reinterpret_cast<CSiteManagerItemData* >(pTree->GetItemData(item));
-	if (!data)
-		XRCCTRL(*this, "wxID_OK", wxButton)->SetFocus();
+	{
+		CSiteManagerItemData* data = 0;
+		wxTreeItemId item = pTree->GetSelection();
+		if (item.IsOk())
+			data = reinterpret_cast<CSiteManagerItemData* >(pTree->GetItemData(item));
+		if (!data)
+			XRCCTRL(*this, "wxID_OK", wxButton)->SetFocus();
+	}
 #endif
 
 	m_connected_sites = connected_sites;
@@ -574,7 +582,7 @@ public:
 		if (!CSiteManager::UnescapeSitePath(lastSelection, m_lastSelection))
 			m_lastSelection.clear();
 		m_wrong_sel_depth = 0;
-		m_kiosk = COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE);
+		m_kiosk = COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE);
 	}
 
 	virtual ~CSiteManagerXmlHandler_Tree()
@@ -871,7 +879,7 @@ bool CSiteManager::Save(TiXmlElement *pElement /*=0*/, wxTreeItemId treeId /*=wx
 		// to the same file or one is reading while the other one writes.
 		CInterProcessMutex mutex(MUTEX_SITEMANAGER);
 
-		wxFileName file(wxGetApp().GetSettingsDir(), _T("sitemanager.xml"));
+		wxFileName file(COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR), _T("sitemanager.xml"));
 		CXmlFile xml(file);
 
 		TiXmlElement* pDocument = xml.Load();
@@ -899,7 +907,7 @@ bool CSiteManager::Save(TiXmlElement *pElement /*=0*/, wxTreeItemId treeId /*=wx
 		wxString error;
 		if (!xml.Save(&error))
 		{
-			if (COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE) == 2)
+			if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
 				return res;
 			wxString msg = wxString::Format(_("Could not write \"%s\", any changes to the Site Manager could not be saved: %s"), file.GetFullPath().c_str(), error.c_str());
 			wxMessageBox(msg, _("Error writing xml file"), wxICON_ERROR);
@@ -1053,7 +1061,7 @@ bool CSiteManager::Verify()
 			return false;
 		}
 
-		if (COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE) != 0 &&
+		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) != 0 &&
 			!IsPredefinedItem(item) &&
 			(logon_type == ACCOUNT || logon_type == NORMAL))
 		{
@@ -2920,7 +2928,7 @@ wxString CSiteManager::AddServer(CServer server)
 	wxString error;
 	if (!file.Save(&error))
 	{
-		if (COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE) == 2)
+		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
 			return _T("");
 
 		wxString msg = wxString::Format(_("Could not write \"%s\", any changes to the Site Manager could not be saved: %s"), file.GetFileName().GetFullPath().c_str(), error.c_str());
@@ -3047,7 +3055,7 @@ bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wx
 	wxString error;
 	if (!file.Save(&error))
 	{
-		if (COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE) == 2)
+		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
 			return true;
 
 		wxString msg = wxString::Format(_("Could not write \"%s\", the selected sites could not be exported: %s"), file.GetFileName().GetFullPath().c_str(), error.c_str());
@@ -3109,7 +3117,7 @@ bool CSiteManager::ClearBookmarks(wxString sitePath)
 	wxString error;
 	if (!file.Save(&error))
 	{
-		if (COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE) == 2)
+		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
 			return true;
 
 		wxString msg = wxString::Format(_("Could not write \"%s\", the selected sites could not be exported: %s"), file.GetFileName().GetFullPath().c_str(), error.c_str());
