@@ -4,6 +4,7 @@
 #include "sizeformatting.h"
 #include "speedlimits_dialog.h"
 #include "statusbar.h"
+#include "themeprovider.h"
 #include "verifycertdialog.h"
 
 static const int statbarWidths[3] = {
@@ -255,6 +256,13 @@ void CWidgetsStatusBar::PositionChildren(int field)
 	}
 }
 
+void CWidgetsStatusBar::SetFieldWidth(int field, int width)
+{
+	wxStatusBarEx::SetFieldWidth(field, width);
+	for (int i = 0; i < GetFieldsCount(); i++)
+		PositionChildren(i);
+}
+
 #ifdef __WXMSW__
 class wxStaticBitmapEx : public wxStaticBitmap
 {
@@ -336,6 +344,9 @@ CStatusBar::CStatusBar(wxTopLevelWindow* pParent)
 
 	RegisterOption(OPTION_ASCIIBINARY);
 
+	// Reload icons
+	RegisterOption(OPTION_THEME);
+
 	CContextManager::Get()->RegisterHandler(this, STATECHANGE_SERVER, true, false);
 	CContextManager::Get()->RegisterHandler(this, STATECHANGE_CHANGEDCONTEXT, false, false);
 
@@ -422,7 +433,7 @@ void CStatusBar::DisplayDataType()
 			desc = _("Current transfer type is set to automatic detection.");
 		}
 
-		wxBitmap bmp = wxArtProvider::GetBitmap(name, wxART_OTHER, wxSize(16, 16));
+		wxBitmap bmp = wxArtProvider::GetBitmap(name, wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall));
 		if (!m_pDataTypeIndicator)
 		{
 			m_pDataTypeIndicator = new CIndicator(this, bmp);
@@ -471,13 +482,15 @@ void CStatusBar::DisplayEncrypted()
 	}
 	else
 	{
-		if (m_pEncryptionIndicator)
-			return;
-		wxBitmap bmp = wxArtProvider::GetBitmap(_T("ART_LOCK"), wxART_OTHER, wxSize(16, 16));
-		m_pEncryptionIndicator = new CIndicator(this, bmp);
-		AddChild(0, widget_encryption, m_pEncryptionIndicator);
-
-		m_pEncryptionIndicator->SetToolTip(_("The connection is encrypted. Click icon for details."));
+		wxBitmap bmp = wxArtProvider::GetBitmap(_T("ART_LOCK"), wxART_OTHER,  CThemeProvider::GetIconSize(iconSizeSmall));
+		if (!m_pEncryptionIndicator)
+		{
+			m_pEncryptionIndicator = new CIndicator(this, bmp);
+			AddChild(0, widget_encryption, m_pEncryptionIndicator);
+			m_pEncryptionIndicator->SetToolTip(_("The connection is encrypted. Click icon for details."));
+		}
+		else
+			m_pEncryptionIndicator->SetBitmap(bmp);
 	}
 }
 
@@ -571,7 +584,7 @@ void CStatusBar::UpdateSpeedLimitsIcon()
 {
 	bool enable = COptions::Get()->GetOptionVal(OPTION_SPEEDLIMIT_ENABLE) != 0;
 
-	wxBitmap bmp = wxArtProvider::GetBitmap(_T("ART_SPEEDLIMITS"), wxART_OTHER, wxSize(16, 16));
+	wxBitmap bmp = wxArtProvider::GetBitmap(_T("ART_SPEEDLIMITS"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall));
 	wxString tooltip;
 
 	int downloadLimit = COptions::Get()->GetOptionVal(OPTION_SPEEDLIMIT_INBOUND);
@@ -604,9 +617,7 @@ void CStatusBar::UpdateSpeedLimitsIcon()
 		AddChild(0, widget_speedlimit, m_pSpeedLimitsIndicator);
 	}
 	else
-	{
 		m_pSpeedLimitsIndicator->SetBitmap(bmp);
-	}
 	m_pSpeedLimitsIndicator->SetToolTip(tooltip);
 }
 
@@ -627,6 +638,11 @@ void CStatusBar::OnOptionChanged(int option)
 	case OPTION_ASCIIBINARY:
 		DisplayDataType();
 		break;
+	case OPTION_THEME:
+		DisplayDataType();
+		UpdateSpeedLimitsIcon();
+		DisplayEncrypted();
+		break;
 	default:
 		break;
 	}
