@@ -9,34 +9,56 @@ CThemeProvider::CThemeProvider()
 	wxArtProvider::Push(this);
 
 	m_themePath = GetThemePath();
+
+	RegisterOption(OPTION_THEME);
 }
 
-wxBitmap CThemeProvider::CreateBitmap(const wxArtID& id, const wxArtClient& client, const wxSize& size)
+static wxString SubdirFromSize(const int size)
+{
+	return wxString::Format(_T("%dx%d/"), size, size);
+}
+
+std::list<wxString> CThemeProvider::GetSearchDirs(const wxSize& size)
+{
+	const int s(size.GetWidth());
+	// Sort order:
+	// - Current theme before general resource dir
+	// - Try current size first
+	// - Then try scale down next larger icon
+	// - Then try scaling up next smaller icon
+
+
+	int sizes[] = { 48,32,24,20,16 };
+
+	std::list<wxString> sizeStrings;
+
+	for (int i = 0; i < sizeof(sizes) / sizeof(int) && sizes[i] > s; ++i)
+		sizeStrings.push_front(SubdirFromSize(sizes[i]));
+	for (int i = 0; i < sizeof(sizes) / sizeof(int); ++i)
+		if (sizes[i] < s)
+			sizeStrings.push_back(SubdirFromSize(sizes[i]));
+
+	sizeStrings.push_front(SubdirFromSize(s));
+
+	std::list<wxString> dirs;
+
+	for (std::list<wxString>::const_iterator it = sizeStrings.begin(); it != sizeStrings.end(); ++it)
+		dirs.push_back(m_themePath + *it);
+
+	const wxString resourceDir(wxGetApp().GetResourceDir());
+	for (std::list<wxString>::const_iterator it = sizeStrings.begin(); it != sizeStrings.end(); ++it)
+		dirs.push_back(resourceDir + *it);
+
+	return dirs;
+}
+
+wxBitmap CThemeProvider::CreateBitmap(const wxArtID& id, const wxArtClient& /*client*/, const wxSize& size)
 {
 	if (id.Left(4) != _T("ART_"))
 		return wxNullBitmap;
+	wxASSERT(size.GetWidth() == size.GetHeight());
 
-	wxString resourceDir = wxGetApp().GetResourceDir();
-
-	std::list<wxString> dirs;
-	wxString strSize = wxString::Format(_T("%dx%d/"), size.GetWidth(), size.GetHeight());
-	if (wxDir::Exists(m_themePath + strSize))
-		dirs.push_back(m_themePath + strSize);
-	if (wxDir::Exists(resourceDir + strSize))
-		dirs.push_back(resourceDir + strSize);
-
-	if (size.GetWidth() >= 40)
-	{
-		dirs.push_back(m_themePath + _T("48x48/"));
-		dirs.push_back(resourceDir + _T("48x48/"));
-	}
-	if (size.GetWidth() >= 24)
-	{
-		dirs.push_back(m_themePath + _T("32x32/"));
-		dirs.push_back(resourceDir + _T("32x32/"));
-	}
-	dirs.push_back(m_themePath + _T("16x16/"));
-	dirs.push_back(resourceDir + _T("16x16/"));
+	std::list<wxString> dirs = GetSearchDirs(size);
 
 	wxString name = id.Mid(4);
 
@@ -51,7 +73,7 @@ wxBitmap CThemeProvider::CreateBitmap(const wxArtID& id, const wxArtClient& clie
 	for (std::list<wxString>::const_iterator iter = dirs.begin(); iter != dirs.end(); iter++)
 	{
 		wxString fileName = *iter + name + _T(".png");
-#ifdef __WXMSW__
+//#ifdef __WXMSW__
 		// MSW toolbar only greys out disabled buttons in a visually
 		// pleasing way if the bitmap has an alpha channel. 
 		wxImage img(fileName, wxBITMAP_TYPE_PNG);
@@ -60,12 +82,14 @@ wxBitmap CThemeProvider::CreateBitmap(const wxArtID& id, const wxArtClient& clie
 
 		if (img.HasMask() && !img.HasAlpha())
 			img.InitAlpha();
+		if (size.IsFullySpecified())
+			img.Rescale(size.x, size.y, wxIMAGE_QUALITY_HIGH);
 		return wxBitmap(img);
-#else
+/*#else
 		wxBitmap bmp(fileName, wxBITMAP_TYPE_PNG);
 		if (bmp.Ok())
 			return bmp;
-#endif
+#endif*/
 	}
 
 	return wxNullBitmap;
@@ -100,9 +124,9 @@ std::list<wxBitmap*> CThemeProvider::GetAllImages(const wxString& theme, const w
 	wxString strSize = wxString::Format(_T("%dx%d/"), size.GetWidth(), size.GetHeight());
 	if (!wxDir::Exists(strSize))
 	{
-		if (size.GetWidth() >= 40)
+		if (size.GetWidth() > 32)
 			path += _T("48x48/");
-		else if (size.GetWidth() >= 24)
+		else if (size.GetWidth() > 16)
 			path += _T("32x32/");
 		else
 			path += _T("16x16/");
@@ -260,3 +284,40 @@ wxString CThemeProvider::GetThemePath()
 	wxASSERT(wxFile::Exists(resourceDir + _T("theme.xml")));
 	return resourceDir;
 }
+
+void CThemeProvider::OnOptionChanged(int option)
+{
+	wxASSERT(option == OPTION_THEME);
+
+	m_themePath = GetThemePath();
+
+	wxArtProvider::Remove(this);
+	wxArtProvider::Push(this);
+}
+
+wxSize CThemeProvider::GetIconSize(enum iconSize size)
+{
+	int s;
+	if (size == iconSizeSmall)
+	{
+		s = wxSystemSettings::GetMetric(wxSYS_SMALLICON_X);
+		if (s <= 0)
+			s = 16;
+	}
+	else if (size == iconSizeLarge)
+	{
+		s = wxSystemSettings::GetMetric(wxSYS_SMALLICON_X);
+		if (s <= 0)
+			s = 48;
+		else
+			s *= 3;
+	}
+	else
+	{
+		s = wxSystemSettings::GetMetric(wxSYS_ICON_X);
+		if (s <= 0)
+			s = 32;
+	}
+
+	return wxSize(s, s);
+}
