@@ -1043,12 +1043,14 @@ int sftp_cmd_quit(struct sftp_command *cmd)
 
 int sftp_cmd_keyfile(struct sftp_command *cmd)
 {
+	Keyfile_list *list;
+
     if (cmd->nwords != 2) {
 	fzprintf(sftpError, "No keyfile given");
 	return 0;
     }
 
-    Keyfile_list *list = snew(Keyfile_list);
+    list = snew(Keyfile_list);
     list->file = filename_from_str(cmd->words[1]);
     list->next = cfg.keyfile_list;
     cfg.keyfile_list = list;
@@ -1364,7 +1366,7 @@ int sftp_general_get(struct sftp_command *cmd, int restart, int multiple)
     /* FZ unused
     while (i < cmd->nwords && cmd->words[i][0] == '-') {
 	if (!strcmp(cmd->words[i], "--")) {
-	    /* finish processing options *//* FZ unused
+	    *//* finish processing options *//* FZ unused
 	    i++;
 	    break;
 	} else if (!strcmp(cmd->words[i], "-r")) {
@@ -2388,7 +2390,7 @@ static struct sftp_cmd_lookup {
 	    "  default location where the \"get\" command will save files).\n",
 	    sftp_cmd_lpwd
     },
-/* END FZ UNUSED */
+END FZ UNUSED */
     {
 	"ls", TRUE, "dir", NULL,
 	    sftp_cmd_ls
@@ -2532,7 +2534,7 @@ static int sftp_cmd_help(struct sftp_command *cmd)
 {
     int i;
     if (cmd->nwords == 1) {
-	/*
+	*//*
 	 * Give short help on each command.
 	 */
 	/* CONTINUE FZ UNUSED
@@ -2557,7 +2559,7 @@ static int sftp_cmd_help(struct sftp_command *cmd)
 	    printf("%s\n", lookup->shorthelp);
 	}
     } else {
-	/*
+	*//*
 	 * Give long help on specific commands.
 	 *//* CONTINUE FZ UNUSED
 	for (i = 1; i < cmd->nwords; i++) {
@@ -2575,7 +2577,7 @@ static int sftp_cmd_help(struct sftp_command *cmd)
     }
     return 1;
 }
-/* END FZ UNUSED */
+END FZ UNUSED */
 
 /* ----------------------------------------------------------------------
  * Command line reading and parsing.
@@ -3318,9 +3320,6 @@ int psftp_init_utf8_locale()
  */
 int psftp_main(int argc, char *argv[])
 {
-    // FZ: Set proxy to none
-    cfg.proxy_type = PROXY_NONE;
-
     int i;
     int portnumber = 0;
     char *userhost, *user;
@@ -3329,6 +3328,9 @@ int psftp_main(int argc, char *argv[])
     char *batchfile = NULL;
     int errors = 0;
 
+    // FZ: Set proxy to none
+    cfg.proxy_type = PROXY_NONE;
+
     fzprintf(sftpReply, "fzSftp started");
 
 #ifndef _WINDOWS
@@ -3414,8 +3416,9 @@ int psftp_main(int argc, char *argv[])
      * it now.
      */
     if (userhost) {
-	fzprintf(sftpVerbose, "psftp: Using userhost passed on commandline: %s", userhost);
 	int ret;
+
+	fzprintf(sftpVerbose, "psftp: Using userhost passed on commandline: %s", userhost);
 	ret = psftp_connect(userhost, user, portnumber);
 	sfree(userhost);
 	if (ret)
