@@ -31,6 +31,20 @@
 # define X11_UNIX_PATH "/tmp/.X11-unix/X"
 #endif
 
+// Union for strict aliasing-safe casting between
+// the different address types
+union sockaddr_u
+{
+#ifdef NO_IPV6
+    struct sockaddr_in addr;
+#else
+    struct sockaddr_storage addr;
+    struct sockaddr_in6 sin6;
+#endif
+    struct sockaddr sa;
+    struct sockaddr_in sin;
+};
+
 /*
  * We used to typedef struct Socket_tag *Socket.
  *
@@ -558,7 +572,7 @@ static int try_connect(Actual_Socket sock)
 	setsockopt(s, SOL_SOCKET, SO_OOBINLINE, (void *) &b, sizeof(b));
     }
 
-    if (sock->nodelay) {
+    /*if (sock->nodelay)*/ {
 	int b = TRUE;
 	setsockopt(s, IPPROTO_TCP, TCP_NODELAY, (void *) &b, sizeof(b));
     }
@@ -568,11 +582,17 @@ static int try_connect(Actual_Socket sock)
 	setsockopt(s, SOL_SOCKET, SO_KEEPALIVE, (void *) &b, sizeof(b));
     }
 
+     /* Enable window scaling */
     {
 	int size_read = 4194304;
 	setsockopt(s, SOL_SOCKET, SO_RCVBUF, (const char*)&size_read, sizeof(size_read));
     }
 
+    {
+	int size_write = 262144;
+	setsockopt(s, SOL_SOCKET, SO_SNDBUF, (const char*)&size_write, sizeof(size_write));
+    }
+
     /*
      * Bind to local address.
      */
@@ -957,14 +977,8 @@ static void sk_tcp_close(Socket sock)
 void *sk_getxdmdata(void *sock, int *lenp)
 {
     Actual_Socket s = (Actual_Socket) sock;
-#ifdef NO_IPV6
-    struct sockaddr_in addr;
-#else
-    struct sockaddr_storage addr;
-    struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *)&addr;
-#endif
-    struct sockaddr *sa = (struct sockaddr *)&addr;
-    struct sockaddr_in *sin = (struct sockaddr_in *)&addr;
+
+    union sockaddr_u addr;
     socklen_t addrlen;
     char *buf;
     static unsigned int unix_addr = 0xFFFFFFFF;
@@ -976,22 +990,22 @@ void *sk_getxdmdata(void *sock, int *lenp)
 	return NULL;		       /* failure */
 
     addrlen = sizeof(addr);
-    if (getsockname(s->s, sa, &addrlen) < 0)
+    if (getsockname(s->s, &addr.sa, &addrlen) < 0)
 	return NULL;
-    switch(sa->sa_family) {
+    switch(addr.sa.sa_family) {
       case AF_INET:
 	*lenp = 6;
 	buf = snewn(*lenp, char);
-	PUT_32BIT_MSB_FIRST(buf, ntohl(sin->sin_addr.s_addr));
-	PUT_16BIT_MSB_FIRST(buf+4, ntohs(sin->sin_port));
+	PUT_32BIT_MSB_FIRST(buf, ntohl(addr.sin.sin_addr.s_addr));
+	PUT_16BIT_MSB_FIRST(buf+4, ntohs(addr.sin.sin_port));
 	break;
 #ifndef NO_IPV6
     case AF_INET6:
 	*lenp = 6;
 	buf = snewn(*lenp, char);
-	if (IN6_IS_ADDR_V4MAPPED(&sin6->sin6_addr)) {
-	    memcpy(buf, sin6->sin6_addr.s6_addr + 12, 4);
-	    PUT_16BIT_MSB_FIRST(buf+4, ntohs(sin6->sin6_port));
+	if (IN6_IS_ADDR_V4MAPPED(&addr.sin6.sin6_addr)) {
+	    memcpy(buf, addr.sin6.sin6_addr.s6_addr + 12, 4);
+	    PUT_16BIT_MSB_FIRST(buf+4, ntohs(addr.sin6.sin6_port));
 	} else
 	    /* This is stupid, but it's what XLib does. */
 	    memset(buf, 0, 6);
@@ -1191,17 +1205,13 @@ static int net_select_result(int fd, int event)
 	     * On a listening socket, the readability event means a
 	     * connection is ready to be accepted.
 	     */
-#ifdef NO_IPV6
-	    struct sockaddr_in ss;
-#else
-	    struct sockaddr_storage ss;
-#endif
+	    union sockaddr_u ss;
 	    socklen_t addrlen = sizeof(ss);
 	    int t;  /* socket of connection */
             int fl;
 
 	    memset(&ss, 0, addrlen);
-	    t = accept(s->s, (struct sockaddr *)&ss, &addrlen);
+	    t = accept(s->s, &ss.sa, &addrlen);
 	    if (t < 0) {
 		break;
 	    }
@@ -1211,7 +1221,7 @@ static int net_select_result(int fd, int event)
                 fcntl(t, F_SETFL, fl | O_NONBLOCK);
 
 	    if (s->localhost_only &&
-		!sockaddr_is_loopback((struct sockaddr *)&ss)) {
+		!sockaddr_is_loopback(&ss.sa)) {
 		close(t);	       /* someone let nonlocal through?! */
 	    } else if (plug_accepting(s->plug, t)) {
 		close(t);	       /* denied or error */
