@@ -884,6 +884,11 @@ static DWORD try_connect(Actual_Socket sock)
 	p_setsockopt(s, SOL_SOCKET, SO_RCVBUF, (const char*)&size_read, sizeof(size_read));
     }
 
+    {
+	int size_write = 262144;
+	p_setsockopt(s, SOL_SOCKET, SO_SNDBUF, (const char*)&size_write, sizeof(size_write));
+    }
+
     /*
      * Bind to local address.
      */
